@@ -97,7 +97,7 @@ typedef enum {
         GNOME_APP_CONFIGURABLE_ITEM_PRINT,
         GNOME_APP_CONFIGURABLE_ITEM_PRINT_SETUP,
         GNOME_APP_CONFIGURABLE_ITEM_CLOSE,
-        GNOME_APP_CONFIGURABLE_ITEM_QUIT,
+        GNOME_APP_CONFIGURABLE_ITEM_EXIT,
         GNOME_APP_CONFIGURABLE_ITEM_CUT,
 	/* 10 */
         GNOME_APP_CONFIGURABLE_ITEM_COPY,
@@ -126,11 +126,6 @@ typedef enum {
 	GNOME_APP_CONFIGURABLE_ITEM_END_GAME
 } GnomeUIInfoConfigurableTypes;
 
-#ifndef GNOME_DISABLE_DEPRECATED
-
-#define GNOME_APP_CONFIGURABLE_ITEM_EXIT	GNOME_APP_CONFIGURABLE_ITEM_QUIT
-
-#endif
 
 /* These values identify the type of pixmap used in an item */
 typedef enum {
@@ -147,8 +142,8 @@ typedef enum {
  * structure is to use the GNOMEUIINFO_* macros provided below. */
 typedef struct {
 	GnomeUIInfoType type;		/* Type of item */
-	gchar const *label;			/* String to use in the label */
-	gchar const *hint;			/* For toolbar items, the tooltip. For 
+	gchar *label;			/* String to use in the label */
+	gchar *hint;			/* For toolbar items, the tooltip. For 
 					   menu items, the status bar message */
 	gpointer moreinfo;		/* For an item, toggleitem, or 
 					   radioitem, this is a pointer to the 
@@ -343,23 +338,11 @@ typedef struct {
           GNOME_APP_PIXMAP_NONE, NULL,                                      \
           GNOME_APP_CONFIGURABLE_ITEM_CLOSE, (GdkModifierType) 0, NULL }
 
-#ifndef GNOME_DISABLE_DEPRECATED
-
-#define GNOMEUIINFO_MENU_EXIT_ITEM(cb, data)	GNOMEUIINFO_MENU_QUIT_ITEM(cb, data)
-
-#endif
-
-#define GNOMEUIINFO_MENU_QUIT_ITEM(cb, data)                                \
+#define GNOMEUIINFO_MENU_EXIT_ITEM(cb, data)                                \
         { GNOME_APP_UI_ITEM_CONFIGURABLE, NULL, NULL,                       \
           (gpointer)cb, (gpointer)(data), NULL,                             \
           GNOME_APP_PIXMAP_NONE, NULL,                                      \
-          GNOME_APP_CONFIGURABLE_ITEM_QUIT, (GdkModifierType) 0, NULL }
-
-#define GNOMEUIINFO_MENU_QUIT_ITEM(cb, data)                                \
-        { GNOME_APP_UI_ITEM_CONFIGURABLE, NULL, NULL,                       \
-          (gpointer)cb, (gpointer)(data), NULL,                             \
-          GNOME_APP_PIXMAP_NONE, NULL,                                      \
-          GNOME_APP_CONFIGURABLE_ITEM_QUIT, (GdkModifierType) 0, NULL }
+          GNOME_APP_CONFIGURABLE_ITEM_EXIT, (GdkModifierType) 0, NULL }
 /*
  * The "Edit" menu
  */
@@ -518,7 +501,7 @@ typedef struct {
 const gchar * gnome_app_helper_gettext (const gchar *string);
 
 #ifdef ENABLE_NLS
-#define D_(x) dgettext (GETTEXT_PACKAGE, x)
+#define D_(x) dgettext (PACKAGE, x)
 #define L_(x) gnome_app_helper_gettext(x)
 #else
 #define D_(x) x
@@ -579,10 +562,7 @@ typedef void (* GnomeUISignalConnectFunc) (GnomeUIInfo        *uiinfo,
 struct _GnomeUIBuilderData {
 	GnomeUISignalConnectFunc connect_func;	/* Function that connects to the item's signals */
 	gpointer data;				/* User data pointer */
-	gboolean is_interp;			/* Should use
-						 * g_signal_connect_closure_by_id
-						 * or normal g_signal_connect?
-						 */
+	gboolean is_interp;			/* Should use gtk_signal_connect_interp or normal gtk_signal_connect? */
 	GtkCallbackMarshal relay_func;		/* Marshaller function for language bindings */
 	GtkDestroyNotify destroy_func;		/* Destroy notification function for language bindings */
 };
