@@ -1,5 +1,5 @@
 /* gnome-appbar.h: statusbar/progress/minibuffer widget for Gnome apps
- *
+ * 
  * This version by Havoc Pennington
  * Based on MozStatusbar widget, by Chris Toshok
  * In turn based on GtkStatusbar widget, from Gtk+,
@@ -26,20 +26,22 @@
  */
 
 #include "config.h"
-#include <libgnome/gnome-macros.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
 #include <string.h> /* for strlen */
 
 #include "gnome-appbar.h"
-#include "gnome-gconf-ui.h"
 
 #include <libgnome/gnome-util.h>
 #include <libgnome/gnome-i18n.h>
+#include <libgnome/gnome-preferences.h>
 
 #include "gnome-uidefs.h"
-#include "gnometypebuiltins.h"
+
+#ifndef GNOME_ENABLE_DEBUG
+#define GNOME_ENABLE_DEBUG /* to be sure */
+#endif
 
 struct _GnomeAppBarPrivate
 {
@@ -47,12 +49,11 @@ struct _GnomeAppBarPrivate
      future. Statusbar could be a label, entry, GtkStatusbar, or
      something else; progress could be a label or progress bar; it's
      all up in the air for now. */
-  /* there is no reason for these not to be properties */
   GtkWidget * progress;
   GtkWidget * status;
   gchar * prompt; /* The text of a prompt, if any. */
 
-  /* Keep it simple; no contexts.
+  /* Keep it simple; no contexts. 
      if (status_stack) display_top_of_stack;
      else if (default_status) display_default;
      else display_nothing;      */
@@ -62,36 +63,91 @@ struct _GnomeAppBarPrivate
 
   gint16 editable_start; /* The first editable position in the interactive
 			  buffer. */
-
-  /* these are construct only currently. there is no reason this
-   * couldn't be changed. */
-
-  GnomePreferencesType interactivity;
   gboolean interactive : 1; /* This means status is an entry rather than a
 			       label, for the moment. */
-  gboolean has_progress : 1;
-  gboolean has_status : 1;
 };
 
+
+static void gnome_appbar_class_init               (GnomeAppBarClass *class);
+static void gnome_appbar_init                     (GnomeAppBar      *ab);
+static void gnome_appbar_destroy                  (GtkObject        *object);
+static void gnome_appbar_finalize                 (GObject          *object);
+     
+static GtkContainerClass *parent_class;
+
 enum {
   USER_RESPONSE,
   CLEAR_PROMPT,
   LAST_SIGNAL
 };
 
-enum {
-	PROP_0,
-	PROP_HAS_PROGRESS,
-	PROP_HAS_STATUS,
-	PROP_INTERACTIVITY
-};
-
 static gint appbar_signals[LAST_SIGNAL] = { 0 };
 
-GNOME_CLASS_BOILERPLATE (GnomeAppBar, gnome_appbar,
-			 GtkHBox, GTK_TYPE_HBOX)
+guint      
+gnome_appbar_get_type ()
+{
+  static guint ab_type = 0;
 
-static GSList *
+  if (!ab_type)
+    {
+      GtkTypeInfo ab_info =
+      {
+        "GnomeAppBar",
+        sizeof (GnomeAppBar),
+        sizeof (GnomeAppBarClass),
+        (GtkClassInitFunc) gnome_appbar_class_init,
+        (GtkObjectInitFunc) gnome_appbar_init,
+        NULL,
+        NULL,
+	NULL
+      };
+
+      ab_type = gtk_type_unique (gtk_hbox_get_type (), &ab_info);
+    }
+
+  return ab_type;
+}
+
+static void
+gnome_appbar_class_init (GnomeAppBarClass *class)
+{
+  GtkObjectClass *object_class;
+  GObjectClass *gobject_class;
+  GtkWidgetClass *widget_class;
+  GtkContainerClass *container_class;
+
+  object_class = (GtkObjectClass *) class;
+  gobject_class = (GObjectClass *) class;
+  widget_class = (GtkWidgetClass *) class;
+  container_class = (GtkContainerClass *) class;
+
+  appbar_signals[USER_RESPONSE] =
+    gtk_signal_new ("user_response",
+		    GTK_RUN_LAST,
+		    GTK_CLASS_TYPE (object_class),
+		    GTK_SIGNAL_OFFSET (GnomeAppBarClass, user_response),
+		    gtk_signal_default_marshaller,
+		    GTK_TYPE_NONE, 0);
+
+  appbar_signals[CLEAR_PROMPT] =
+    gtk_signal_new ("clear_prompt",
+		    GTK_RUN_LAST,
+		    GTK_CLASS_TYPE (object_class),
+		    GTK_SIGNAL_OFFSET (GnomeAppBarClass, clear_prompt),
+		    gtk_signal_default_marshaller,
+		    GTK_TYPE_NONE, 0);
+
+  parent_class = gtk_type_class (gtk_hbox_get_type ());
+  
+  class->user_response = NULL;
+  class->clear_prompt  = NULL; /* maybe should have a handler
+				  and the clear_prompt function
+				  just emits. */
+  object_class->destroy = gnome_appbar_destroy;
+  gobject_class->finalize = gnome_appbar_finalize;
+}
+
+static GSList * 
 stringstack_push(GSList * stringstack, const gchar * s)
 {
   return g_slist_prepend(stringstack, g_strdup(s));
@@ -127,41 +183,52 @@ stringstack_free(GSList * stringstack)
 }
 
 static void
-entry_delete_text_cb(GtkWidget * entry, gint start,
+entry_delete_text_cb(GtkWidget * entry, gint start, 
 		     gint stop, GnomeAppBar * ab)
 {
   g_return_if_fail(GNOME_IS_APPBAR(ab));
   g_return_if_fail(GTK_IS_ENTRY(entry));
   g_return_if_fail(ab->_priv->interactive);
 
-  if (ab->_priv->prompt == NULL) return; /* not prompting, so don't
+  if (ab->_priv->prompt == NULL) return; /* not prompting, so don't 
 				     interfere. */
+#ifdef GNOME_ENABLE_DEBUG
+    g_print("Start is %d, stop is %d, start of editable is %d\n", 
+	    start, stop, ab->_priv->editable_start);
+#endif
+  
   if (start < ab->_priv->editable_start) {
     /* Block the signal, since it's trying to delete text
        that shouldn't be deleted. */
-    g_signal_stop_emission_by_name(entry, "delete_text");
+    gtk_signal_emit_stop_by_name(GTK_OBJECT(entry), "delete_text");
     gdk_beep();
   }
 }
-static void
+static void 
 entry_insert_text_cb  (GtkEditable    *entry,
 		       const gchar    *text,
 		       gint            length,
 		       gint           *position,
 		       GnomeAppBar * ab)
 {
-  gint pos;
+  gint pos; 
 
-  if (ab->_priv->prompt == NULL) return; /* not prompting, so don't
+  if (ab->_priv->prompt == NULL) return; /* not prompting, so don't 
 				     interfere. */
 
   pos = gtk_editable_get_position(entry);
 
+#ifdef GNOME_ENABLE_DEBUG
+    g_print("Position is %d, start of editable is %d\n", 
+	    pos, ab->_priv->editable_start);
+#endif
+
   if (pos < ab->_priv->editable_start) {
 
     /*    gtk_editable_set_position(entry, ab->_priv->editable_start); */
-    g_signal_stop_emission_by_name(entry, "insert_text");
-    /*    g_signal_emit_by_name(entry, "insert_text", text, length, position); */
+    gtk_signal_emit_stop_by_name(GTK_OBJECT(entry), "insert_text");
+    /*    gtk_signal_emit_by_name(GTK_OBJECT(entry), "insert_text",
+			    text, length, position); */
   }
 }
 
@@ -170,7 +237,7 @@ entry_key_press_cb(GtkWidget * entry, GdkEventKey * e, GnomeAppBar * ab)
 {
   /* This should be some kind of configurable binding
       or accelerator, probably. */
-  if ( (e->keyval == GDK_g) &&
+  if ( (e->keyval == GDK_g) && 
        (e->state & GDK_CONTROL_MASK) ) {
     /* Cancel */
     gnome_appbar_clear_prompt(ab);
@@ -182,11 +249,11 @@ entry_key_press_cb(GtkWidget * entry, GdkEventKey * e, GnomeAppBar * ab)
 static void
 entry_activate_cb(GtkWidget * entry, GnomeAppBar * ab)
 {
-  g_signal_emit(ab, appbar_signals[USER_RESPONSE], 0);
+  gtk_signal_emit(GTK_OBJECT(ab), appbar_signals[USER_RESPONSE]);
 }
 
 static void
-gnome_appbar_instance_init (GnomeAppBar *ab)
+gnome_appbar_init (GnomeAppBar *ab)
 {
   ab->_priv                 = g_new0(GnomeAppBarPrivate, 1);
   ab->_priv->default_status = NULL;
@@ -225,18 +292,120 @@ gnome_appbar_instance_init (GnomeAppBar *ab)
  * Returns:  Pointer to new GNOME appbar widget.
  **/
 
-GtkWidget*
+GtkWidget* 
 gnome_appbar_new (gboolean has_progress,
 		  gboolean has_status,
 		  GnomePreferencesType interactivity)
 {
-  return GTK_WIDGET (g_object_new (GNOME_TYPE_APPBAR,
-				   "has_progress", has_progress,
-				   "has_status", has_status,
-				   "interactivity", interactivity,
-				   NULL));
+  GnomeAppBar * ab = gtk_type_new (gnome_appbar_get_type ());
+
+  gnome_appbar_construct(ab, has_progress, has_status, interactivity);
+
+  return GTK_WIDGET(ab);
 }
 
+
+/**
+ * gnome_appbar_construct
+ * @ab: Pointer to GNOME appbar object.
+ * @has_progress: %TRUE if appbar needs progress bar widget.
+ * @has_status: %TRUE if appbar needs status bar widget.
+ * @interactivity: See gnome_appbar_new() explanation.
+ *
+ * Description:
+ * For use to bindings in languages other than C. Don't use.
+ **/
+
+void
+gnome_appbar_construct(GnomeAppBar * ab,
+		       gboolean has_progress,
+		       gboolean has_status,
+		       GnomePreferencesType interactivity)
+{
+  GtkBox *box;
+
+  /* These checks are kind of gross because an unfinished object will
+     be returned from _new instead of NULL */
+
+  /* Can't be interactive if there's no status bar */
+  g_return_if_fail( ((has_status == FALSE) && 
+		     (interactivity == GNOME_PREFERENCES_NEVER)) ||
+		    (has_status == TRUE)); 
+
+  box = GTK_BOX (ab);
+
+  box->spacing = GNOME_PAD_SMALL;
+  box->homogeneous = FALSE;
+
+  if (has_progress)
+    ab->_priv->progress = gtk_progress_bar_new();
+  else
+    ab->_priv->progress = NULL;
+
+  /*
+   * If the progress meter goes on the right then we place it after we
+   * create the status line.
+   */
+  if (has_progress && !gnome_preferences_get_statusbar_meter_on_right ())
+    gtk_box_pack_start (box, ab->_priv->progress, FALSE, FALSE, 0);
+
+  if ( has_status ) {
+    if ( (interactivity == GNOME_PREFERENCES_ALWAYS) ||
+	 ( (interactivity == GNOME_PREFERENCES_USER) &&
+	   gnome_preferences_get_statusbar_interactive()) ) {
+      ab->_priv->interactive = TRUE;
+   
+      ab->_priv->status = gtk_entry_new();
+
+      gtk_signal_connect (GTK_OBJECT(ab->_priv->status), "delete_text",
+			  GTK_SIGNAL_FUNC(entry_delete_text_cb),
+			  ab);
+      gtk_signal_connect (GTK_OBJECT(ab->_priv->status), "insert_text",
+			  GTK_SIGNAL_FUNC(entry_insert_text_cb),
+			  ab);
+      gtk_signal_connect_after(GTK_OBJECT(ab->_priv->status), "key_press_event",
+			       GTK_SIGNAL_FUNC(entry_key_press_cb),
+			       ab);
+      gtk_signal_connect(GTK_OBJECT(ab->_priv->status), "activate",
+			 GTK_SIGNAL_FUNC(entry_activate_cb),
+			 ab);
+
+      /* no prompt now */
+      gtk_entry_set_editable(GTK_ENTRY(ab->_priv->status), FALSE);
+
+      gtk_box_pack_start (box, ab->_priv->status, TRUE, TRUE, 0);
+    }
+    else {
+      GtkWidget * frame;
+      
+      ab->_priv->interactive = FALSE;
+
+      frame = gtk_frame_new (NULL);
+      gtk_frame_set_shadow_type (GTK_FRAME(frame), GTK_SHADOW_IN);
+      
+      ab->_priv->status = gtk_label_new ("");
+      gtk_misc_set_alignment (GTK_MISC (ab->_priv->status), 0.0, 0.0);
+      gtk_widget_set_usize (ab->_priv->status, 1, -1);
+      
+      gtk_box_pack_start (box, frame, TRUE, TRUE, 0);
+      gtk_container_add (GTK_CONTAINER(frame), ab->_priv->status);
+      
+      gtk_widget_show (frame);
+    }
+  }
+  else {
+    ab->_priv->status = NULL;
+    ab->_priv->interactive = FALSE;
+  }
+
+  if (has_progress && gnome_preferences_get_statusbar_meter_on_right ())
+    gtk_box_pack_start (box, ab->_priv->progress, FALSE, FALSE, 0);
+
+  if (ab->_priv->status) gtk_widget_show (ab->_priv->status);
+  if (ab->_priv->progress) gtk_widget_show(ab->_priv->progress);
+}
+
+
 /**
  * gnome_appbar_set_prompt
  * @appbar: Pointer to GNOME appbar object.
@@ -249,7 +418,7 @@ gnome_appbar_new (gboolean has_progress,
  **/
 
 void
-gnome_appbar_set_prompt (GnomeAppBar * appbar,
+gnome_appbar_set_prompt (GnomeAppBar * appbar, 
 			 const gchar * prompt,
 			 gboolean modal)
 {
@@ -278,7 +447,7 @@ gnome_appbar_set_prompt (GnomeAppBar * appbar,
  * Remove any prompt.
  **/
 
-void
+void       
 gnome_appbar_clear_prompt    (GnomeAppBar * appbar)
 {
   g_return_if_fail(appbar != NULL);
@@ -286,13 +455,14 @@ gnome_appbar_clear_prompt    (GnomeAppBar * appbar)
 
   /* This isn't really right; most of Gtk would only emit here,
      and then have a clear_prompt_real as default handler. */
-
+  
   g_free(appbar->_priv->prompt);
   appbar->_priv->prompt = NULL;
 
   gnome_appbar_refresh(appbar);
 
-  g_signal_emit (appbar, appbar_signals[CLEAR_PROMPT], 0);
+  gtk_signal_emit(GTK_OBJECT(appbar), 
+		  appbar_signals[CLEAR_PROMPT]);  
 }
 
 
@@ -307,17 +477,18 @@ gnome_appbar_clear_prompt    (GnomeAppBar * appbar)
  * Text from appbar entry widget, as entered by user.
  **/
 
-gchar *
+gchar *    
 gnome_appbar_get_response    (GnomeAppBar * appbar)
 {
   g_return_val_if_fail(appbar != NULL, NULL);
   g_return_val_if_fail(appbar->_priv->interactive, NULL);
-
+  g_return_val_if_fail(appbar->_priv->prompt != NULL, NULL);
+  
   /* This returns an allocated string. */
   return gtk_editable_get_chars(GTK_EDITABLE(appbar->_priv->status),
-				appbar->_priv->editable_start,
+				appbar->_priv->editable_start, 
 				GTK_ENTRY(appbar->_priv->status)->text_length);
-}
+} 
 
 
 /**
@@ -329,28 +500,28 @@ gnome_appbar_get_response    (GnomeAppBar * appbar)
  * set_status to disappear.
  **/
 
-void
+void 
 gnome_appbar_refresh           (GnomeAppBar * appbar)
 {
   g_return_if_fail(appbar != NULL);
   g_return_if_fail(GNOME_IS_APPBAR(appbar));
-
+  
   if (appbar->_priv->prompt) {
     g_return_if_fail(appbar->_priv->interactive); /* Just a consistency check */
-    gtk_editable_set_editable(GTK_EDITABLE(appbar->_priv->status), TRUE);
+    gtk_entry_set_editable(GTK_ENTRY(appbar->_priv->status), TRUE);
     /* Allow insert_text to work, so we can set the prompt. */
     appbar->_priv->editable_start = 0;
     gtk_entry_set_text(GTK_ENTRY(appbar->_priv->status), appbar->_priv->prompt);
     /* This has to be after setting the text. */
-    appbar->_priv->editable_start = strlen(appbar->_priv->prompt);
-    gtk_editable_set_position(GTK_EDITABLE(appbar->_priv->status),
-			      appbar->_priv->editable_start);
+    appbar->_priv->editable_start = strlen(appbar->_priv->prompt);   
+    gtk_entry_set_position(GTK_ENTRY(appbar->_priv->status), 
+			   appbar->_priv->editable_start);
     gtk_widget_grab_focus(appbar->_priv->status);
   }
   else {
     if (appbar->_priv->interactive) {
       appbar->_priv->editable_start = 0;
-      gtk_editable_set_editable(GTK_EDITABLE(appbar->_priv->status), FALSE);
+      gtk_entry_set_editable(GTK_ENTRY(appbar->_priv->status), FALSE);
       gtk_grab_remove(appbar->_priv->status); /* In case */
     }
 
@@ -358,7 +529,7 @@ gnome_appbar_refresh           (GnomeAppBar * appbar)
       gnome_appbar_set_status(appbar, stringstack_top(appbar->_priv->status_stack));
     else if (appbar->_priv->default_status)
       gnome_appbar_set_status(appbar, appbar->_priv->default_status);
-    else
+    else 
       gnome_appbar_set_status(appbar, "");
   }
 }
@@ -371,10 +542,10 @@ gnome_appbar_refresh           (GnomeAppBar * appbar)
  *
  * Description:
  * Sets the status label without changing widget state; next set or push
- * will destroy this permanently.
+ * will destroy this permanently. 
  **/
 
-void
+void       
 gnome_appbar_set_status       (GnomeAppBar * appbar,
 			       const gchar * status)
 {
@@ -382,7 +553,7 @@ gnome_appbar_set_status       (GnomeAppBar * appbar,
   g_return_if_fail(status != NULL);
   g_return_if_fail(GNOME_IS_APPBAR(appbar));
 
-  if (appbar->_priv->interactive)
+  if (appbar->_priv->interactive) 
     gtk_entry_set_text(GTK_ENTRY(appbar->_priv->status), status);
   else
     gtk_label_set_text(GTK_LABEL(appbar->_priv->status), status);
@@ -398,14 +569,14 @@ gnome_appbar_set_status       (GnomeAppBar * appbar,
  * What to show when showing nothing else; defaults to nothing.
  **/
 
-void
+void	   
 gnome_appbar_set_default      (GnomeAppBar * appbar,
 			       const gchar * default_status)
 {
   g_return_if_fail(appbar != NULL);
   g_return_if_fail(default_status != NULL);
   g_return_if_fail(GNOME_IS_APPBAR(appbar));
-
+  
   /* g_free handles NULL */
   g_free(appbar->_priv->default_status);
   appbar->_priv->default_status = g_strdup(default_status);
@@ -423,7 +594,7 @@ gnome_appbar_set_default      (GnomeAppBar * appbar,
  * display it.
  **/
 
-void
+void       
 gnome_appbar_push             (GnomeAppBar * appbar,
 			       const gchar * status)
 {
@@ -445,12 +616,12 @@ gnome_appbar_push             (GnomeAppBar * appbar,
  * message, if any.  It is OK to call this with an empty stack.
  **/
 
-void
+void       
 gnome_appbar_pop              (GnomeAppBar * appbar)
 {
   g_return_if_fail(appbar != NULL);
   g_return_if_fail(GNOME_IS_APPBAR(appbar));
-
+  
   appbar->_priv->status_stack = stringstack_pop(appbar->_priv->status_stack);
   gnome_appbar_refresh(appbar);
 }
@@ -465,7 +636,7 @@ gnome_appbar_pop              (GnomeAppBar * appbar)
  * message (if present).
  **/
 
-void
+void       
 gnome_appbar_clear_stack      (GnomeAppBar * appbar)
 {
   g_return_if_fail(appbar != NULL);
@@ -494,7 +665,7 @@ gnome_appbar_set_progress_percentage(GnomeAppBar *appbar,
   g_return_if_fail (appbar->_priv->progress != NULL);
   g_return_if_fail (GNOME_IS_APPBAR(appbar));
 
-  gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(appbar->_priv->progress), percentage);
+  gtk_progress_bar_update(GTK_PROGRESS_BAR(appbar->_priv->progress), percentage);
 }
 
 
@@ -503,7 +674,7 @@ gnome_appbar_set_progress_percentage(GnomeAppBar *appbar,
  * @appbar: Pointer to GNOME appbar object
  *
  * Description:
- * Returns &GtkProgressBar widget pointer, so that the progress bar may be
+ * Returns &GtkProgress widget pointer, so that the progress bar may be
  * manipulated further.
  *
  * Returns:
@@ -511,14 +682,14 @@ gnome_appbar_set_progress_percentage(GnomeAppBar *appbar,
  * has no progress object.
  **/
 
-GtkProgressBar*
+GtkProgress* 
 gnome_appbar_get_progress    (GnomeAppBar * appbar)
 {
   g_return_val_if_fail(appbar != NULL, NULL);
   g_return_val_if_fail(GNOME_IS_APPBAR(appbar), NULL);
   g_return_val_if_fail(appbar->_priv->progress != NULL, NULL);
 
-  return GTK_PROGRESS_BAR(appbar->_priv->progress);
+  return GTK_PROGRESS(appbar->_priv->progress);
 }
 
 /**
@@ -540,6 +711,35 @@ gnome_appbar_get_status    (GnomeAppBar * appbar)
   return (appbar->_priv->status);
 }
 
+static void
+gnome_appbar_destroy (GtkObject *object)
+{
+  GnomeAppBar *ab;
+  GnomeAppBarClass *class;
+
+  /* remember, destroy can be run multiple times! */
+
+  g_return_if_fail (object != NULL);
+  g_return_if_fail (GNOME_IS_APPBAR (object));
+
+  ab = GNOME_APPBAR (object);
+  class = GNOME_APPBAR_GET_CLASS (ab);
+
+  if(ab->_priv->status_stack) {
+	  stringstack_free(ab->_priv->status_stack);
+	  ab->_priv->status_stack = NULL;
+  }
+
+  /* g_free checks if these are NULL */
+  g_free(ab->_priv->default_status);
+  ab->_priv->default_status = NULL;;
+  g_free(ab->_priv->prompt);
+  ab->_priv->prompt = NULL;
+
+  if(GTK_OBJECT_CLASS (parent_class)->destroy)
+	  GTK_OBJECT_CLASS (parent_class)->destroy (object);
+}
+
 static void
 gnome_appbar_finalize (GObject *object)
 {
@@ -550,225 +750,10 @@ gnome_appbar_finalize (GObject *object)
 
   ab = GNOME_APPBAR (object);
 
-  /* stringstack_free handles null correctly */
-  stringstack_free (ab->_priv->status_stack);
-  ab->_priv->status_stack = NULL;
-
-  /* g_free checks if these are NULL */
-  g_free (ab->_priv->default_status);
-  ab->_priv->default_status = NULL;
-  g_free (ab->_priv->prompt);
-  ab->_priv->prompt = NULL;
-
-  g_free (ab->_priv);
+  g_free(ab->_priv);
   ab->_priv = NULL;
 
-  GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
+  if(G_OBJECT_CLASS (parent_class)->finalize)
+	  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static GObject *
-gnome_appbar_constructor (GType                  type,
-			  guint                  n_properties,
-			  GObjectConstructParam *properties)
-{
-  GObject *object;
-  GnomeAppBar *ab;
-  GtkBox *box;
-  gboolean has_status, has_progress, interactive;
-
-  object = G_OBJECT_CLASS (parent_class)->constructor (type,
-						       n_properties,
-						       properties);
-
-  ab = GNOME_APPBAR (object);
-
-  has_status   = ab->_priv->has_status;
-  has_progress = ab->_priv->has_progress;
-  interactive  = ab->_priv->interactive;
-
-  box = GTK_BOX (ab);
-
-  box->spacing = GNOME_PAD_SMALL;
-  box->homogeneous = FALSE;
-
-  if (has_progress)
-    ab->_priv->progress = gtk_progress_bar_new ();
-
-  /*
-   * If the progress meter goes on the right then we place it after we
-   * create the status line.
-   */
-  if (has_progress &&
-      /* FIXME: this should listen to changes! */
-      ! gnome_gconf_get_bool ("/desktop/gnome/interface/statusbar-meter-on-right"))
-    gtk_box_pack_start (box, ab->_priv->progress, FALSE, FALSE, 0);
-
-  if ( has_status ) {
-    if ( interactive ) {
-      ab->_priv->status = gtk_entry_new();
-
-      g_signal_connect (ab->_priv->status, "delete_text",
-			G_CALLBACK(entry_delete_text_cb),
-			ab);
-      g_signal_connect (ab->_priv->status, "insert_text",
-			G_CALLBACK(entry_insert_text_cb),
-			ab);
-      g_signal_connect_after (ab->_priv->status, "key_press_event",
-			      G_CALLBACK(entry_key_press_cb),
-			      ab);
-      g_signal_connect (ab->_priv->status, "activate",
-			G_CALLBACK(entry_activate_cb),
-			ab);
-
-      /* no prompt now */
-      gtk_editable_set_editable(GTK_EDITABLE(ab->_priv->status), FALSE);
-
-      gtk_box_pack_start (box, ab->_priv->status, TRUE, TRUE, 0);
-    } else {
-      GtkWidget * frame;
-
-      frame = gtk_frame_new (NULL);
-      gtk_frame_set_shadow_type (GTK_FRAME(frame), GTK_SHADOW_IN);
-
-      ab->_priv->status = gtk_label_new ("");
-      gtk_misc_set_alignment (GTK_MISC (ab->_priv->status), 0.0, 0.0);
-      gtk_widget_set_size_request (ab->_priv->status, 1, -1);
-
-      gtk_box_pack_start (box, frame, TRUE, TRUE, 0);
-      gtk_container_add (GTK_CONTAINER(frame), ab->_priv->status);
-
-      gtk_widget_show (frame);
-    }
-  }
-
-  if (has_progress &&
-      /* FIXME: this should listen to changes! */
-      gnome_gconf_get_bool ("/desktop/gnome/interface/statusbar-meter-on-right"))
-    gtk_box_pack_start (box, ab->_priv->progress, FALSE, FALSE, 0);
-
-  if (ab->_priv->status) gtk_widget_show (ab->_priv->status);
-  if (ab->_priv->progress) gtk_widget_show(ab->_priv->progress);
-
-  return object;
-}
-
-static void
-gnome_appbar_set_property (GObject      *object,
-			   guint         prop_id,
-			   const GValue *value,
-			   GParamSpec   *pspec)
-{
-  GnomeAppBarPrivate *priv = GNOME_APPBAR (object)->_priv;
-  switch (prop_id) {
-  case PROP_HAS_STATUS:
-    priv->has_status = g_value_get_boolean (value);
-    break;
-  case PROP_HAS_PROGRESS:
-    priv->has_progress = g_value_get_boolean (value);
-    break;
-  case PROP_INTERACTIVITY:
-    priv->interactivity = g_value_get_enum (value);
-    switch (priv->interactivity) {
-    case GNOME_PREFERENCES_NEVER:
-      priv->interactive = FALSE;
-      break;
-    case GNOME_PREFERENCES_ALWAYS:
-      priv->interactive = TRUE;
-      break;
-    default:
-      priv->interactive = gnome_gconf_get_bool ("/desktop/gnome/interface/statusbar-interactive");
-      break;
-    }
-  }
-}
-
-static void
-gnome_appbar_get_property (GObject    *object,
-			   guint       prop_id,
-			   GValue     *value,
-			   GParamSpec *pspec)
-{
-  GnomeAppBarPrivate *priv = GNOME_APPBAR (object)->_priv;
-
-  switch (prop_id) {
-  case PROP_HAS_STATUS:
-    g_value_set_boolean (value, priv->has_status);
-    break;
-  case PROP_HAS_PROGRESS:
-    g_value_set_boolean (value, priv->has_progress);
-    break;
-  case PROP_INTERACTIVITY:
-    g_value_set_enum (value, priv->interactivity);
-    break;
-  }
-}
-
-static void
-gnome_appbar_class_init (GnomeAppBarClass *class)
-{
-  GtkObjectClass *object_class;
-  GObjectClass *gobject_class;
-  GtkWidgetClass *widget_class;
-  GtkContainerClass *container_class;
-
-  object_class = (GtkObjectClass *) class;
-  gobject_class = (GObjectClass *) class;
-  widget_class = (GtkWidgetClass *) class;
-  container_class = (GtkContainerClass *) class;
-
-  parent_class = GTK_HBOX_CLASS (gtk_type_class (GTK_TYPE_HBOX));
-
-  gobject_class->constructor  = gnome_appbar_constructor;
-  gobject_class->get_property = gnome_appbar_get_property;
-  gobject_class->set_property = gnome_appbar_set_property;
-
-  appbar_signals[USER_RESPONSE] =
-    g_signal_new ("user_response",
-		  G_TYPE_FROM_CLASS (gobject_class),
-		  G_SIGNAL_RUN_LAST,
-		  G_STRUCT_OFFSET (GnomeAppBarClass, user_response),
-		  NULL, NULL,
-		  g_cclosure_marshal_VOID__VOID,
-		  G_TYPE_NONE, 0);
-
-  appbar_signals[CLEAR_PROMPT] =
-    g_signal_new ("clear_prompt",
-		  G_TYPE_FROM_CLASS (gobject_class),
-		  G_SIGNAL_RUN_LAST,
-		  G_STRUCT_OFFSET (GnomeAppBarClass, clear_prompt),
-		  NULL, NULL,
-		  g_cclosure_marshal_VOID__VOID,
-		  G_TYPE_NONE, 0);
-
-  g_object_class_install_property (
-	  gobject_class,
-	  PROP_HAS_PROGRESS,
-	  g_param_spec_boolean ("has_progress",
-				_("Has Progress"),
-				_("Create a progress widget."),
-				FALSE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-
-  g_object_class_install_property (
-	  gobject_class,
-	  PROP_HAS_STATUS,
-	  g_param_spec_boolean ("has_status",
-				_("Has Status"),
-				_("Create a status widget."),
-				FALSE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-
-  g_object_class_install_property (
-	  gobject_class,
-	  PROP_INTERACTIVITY,
-	  g_param_spec_enum ("interactivity",
-			     _("Interactivity"),
-			     _("Level of user activity required."),
-			     GNOME_TYPE_PREFERENCES_TYPE,
-			     GNOME_PREFERENCES_NEVER,
-			     G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-
-  class->user_response = NULL;
-  class->clear_prompt  = NULL; /* maybe should have a handler
-				  and the clear_prompt function
-				  just emits. */
-  gobject_class->finalize = gnome_appbar_finalize;
-}
