@@ -29,7 +29,7 @@
 #include <gtk/gtkwindow.h>
 #include <gtk/gtkeditable.h>
 #include <gtk/gtkaccelgroup.h>
-#include <gtk/gtkstock.h>
+
 
 #include <stdarg.h>
 
@@ -43,6 +43,7 @@ G_BEGIN_DECLS
 #define GNOME_DIALOG_GET_CLASS(obj)  (GTK_CHECK_GET_CLASS ((obj), GNOME_TYPE_DIALOG, GnomeDialogClass))
 
 typedef struct _GnomeDialog        GnomeDialog;
+typedef struct _GnomeDialogPrivate GnomeDialogPrivate;
 typedef struct _GnomeDialogClass   GnomeDialogClass;
 
 /* The vbox can be accessed directly; if you fool with anything else,
@@ -52,14 +53,9 @@ struct _GnomeDialog
   GtkWindow window;
   /*< public >*/
   GtkWidget * vbox;
-  /*< private >*/
   GList *buttons;
-  GtkWidget      *action_area; /* A button box, not an hbox */
-
-  GtkAccelGroup  *accelerators;
-
-  unsigned int    click_closes : 1;
-  unsigned int    just_hide : 1;
+  /*< private >*/
+  GnomeDialogPrivate *_priv;
 };
 
 struct _GnomeDialogClass
@@ -76,7 +72,7 @@ struct _GnomeDialogClass
    order you passed them in, starting at 0. These numbers are used
    in other functions, and passed to the "clicked" callback. */
 
-GType      gnome_dialog_get_type       (void) G_GNUC_CONST;
+guint      gnome_dialog_get_type       (void) G_GNUC_CONST;
 
 /* Arguments: Title and button names, then NULL */
 GtkWidget* gnome_dialog_new            (const gchar * title,
@@ -96,12 +92,12 @@ void       gnome_dialog_set_parent     (GnomeDialog * dialog,
 /* Connect to the "clicked" signal of a single button */
 void       gnome_dialog_button_connect (GnomeDialog *dialog,
 					gint button,
-					GCallback callback,
+					GtkSignalFunc callback,
 					gpointer data);
 /* Connect the object to the "clicked" signal of a single button */
 void       gnome_dialog_button_connect_object (GnomeDialog *dialog,
 					       gint button,
-					       GCallback callback,
+					       GtkSignalFunc callback,
 					       GtkObject * obj);
 
 /* Run the dialog, return the button # that was pressed or -1 if none.
