@@ -37,9 +37,6 @@
 
 
 #include <libgnomecanvas/gnome-canvas.h>
-#include <libgnomecanvas/gnome-canvas-rich-text.h>
-#include <libgnomecanvas/gnome-canvas-pixbuf.h>
-#include <libgnomeui/gnome-icon-item.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
 G_BEGIN_DECLS
@@ -69,10 +66,6 @@ typedef enum {
 struct _GnomeIconList {
 	GnomeCanvas canvas;
 
-	/* Scroll adjustments */
-	GtkAdjustment *adj;
-	GtkAdjustment *hadj;
-	
 	/*< private >*/
 	GnomeIconListPrivate * _priv;
 };
@@ -80,19 +73,9 @@ struct _GnomeIconList {
 struct _GnomeIconListClass {
 	GnomeCanvasClass parent_class;
 
-	void     (*select_icon)      (GnomeIconList *gil, gint num, GdkEvent *event);
-	void     (*unselect_icon)    (GnomeIconList *gil, gint num, GdkEvent *event);
-	void     (*focus_icon)       (GnomeIconList *gil, gint num);
-	gboolean (*text_changed)     (GnomeIconList *gil, gint num, const char *new_text);
-
-	/* Key Binding signals */
-	void     (*move_cursor)      (GnomeIconList *gil, GtkDirectionType dir, gboolean clear_selection);
-	void     (*toggle_cursor_selection) (GnomeIconList *gil);
-	void     (*unused)       (GnomeIconList *unused);
-	
-	/* Padding for possible expansion */
-	gpointer padding1;
-	gpointer padding2;
+	void     (*select_icon)    (GnomeIconList *gil, gint num, GdkEvent *event);
+	void     (*unselect_icon)  (GnomeIconList *gil, gint num, GdkEvent *event);
+	gboolean (*text_changed)   (GnomeIconList *gil, gint num, const char *new_text);
 };
 
 enum {
@@ -100,22 +83,14 @@ enum {
 	GNOME_ICON_LIST_STATIC_TEXT	= 1 << 1
 };
 
-GType          gnome_icon_list_get_type            (void) G_GNUC_CONST;
+guint          gnome_icon_list_get_type            (void) G_GNUC_CONST;
 
 GtkWidget     *gnome_icon_list_new                 (guint         icon_width,
-						    GtkAdjustment *adj,
 						    int           flags);
-
 void           gnome_icon_list_construct           (GnomeIconList *gil,
 						    guint icon_width,
-						    GtkAdjustment *adj,
 						    int flags);
 
-void           gnome_icon_list_set_hadjustment    (GnomeIconList *gil,
-						   GtkAdjustment *hadj);
-
-void           gnome_icon_list_set_vadjustment    (GnomeIconList *gil,
-						   GtkAdjustment *vadj);
 
 /* To avoid excesive recomputes during insertion/deletion */
 void           gnome_icon_list_freeze              (GnomeIconList *gil);
@@ -158,10 +133,6 @@ void           gnome_icon_list_unselect_icon       (GnomeIconList *gil,
 int            gnome_icon_list_unselect_all        (GnomeIconList *gil);
 GList *        gnome_icon_list_get_selection       (GnomeIconList *gil);
 
-/* Managing focus */
-void           gnome_icon_list_focus_icon          (GnomeIconList *gil,
-						    gint idx);
-
 /* Setting the spacing values */
 void           gnome_icon_list_set_icon_width      (GnomeIconList *gil,
 						    int w);
@@ -186,7 +157,7 @@ void           gnome_icon_list_set_icon_data       (GnomeIconList *gil,
 						    int idx, gpointer data);
 void           gnome_icon_list_set_icon_data_full  (GnomeIconList *gil,
 						    int idx, gpointer data,
-						    GDestroyNotify destroy);
+						    GtkDestroyNotify destroy);
 int            gnome_icon_list_find_icon_from_data (GnomeIconList *gil,
 						    gpointer data);
 gpointer       gnome_icon_list_get_icon_data       (GnomeIconList *gil,
@@ -203,15 +174,6 @@ int            gnome_icon_list_get_icon_at         (GnomeIconList *gil,
 
 int            gnome_icon_list_get_items_per_line  (GnomeIconList *gil);
 
-/* Accessibility functions */
-GnomeIconTextItem *gnome_icon_list_get_icon_text_item (GnomeIconList *gil,
-						       int idx);
-
-GnomeCanvasPixbuf *gnome_icon_list_get_icon_pixbuf_item (GnomeIconList *gil,
-							 int idx);
-
-
-
 G_END_DECLS
 
 #endif /* _GNOME_ICON_LIST_H_ */
