@@ -23,7 +23,7 @@
 */
 
 #include <config.h>
-#include <libgnome/gnome-macros.h>
+#include "gnome-macros.h"
 
 #include <unistd.h> 
 #include <sys/stat.h>
@@ -40,25 +40,23 @@
 
 #include "gnome-icon-sel.h"
 
-#include <libgnomevfs/gnome-vfs-ops.h>
+#include <libgnomevfs/gnome-vfs-mime.h>
 
 #define ICON_SIZE 48
 
 struct _GnomeIconSelectionPrivate {
-	GtkWidget * box;
+  GtkWidget * box;
 
-	GtkWidget * gil;
+  GtkWidget * gil;
 
-	GList * file_list;
-
-	/* For loading the icons */
-	GMainLoop * load_loop;
-	guint load_idle;
-	int load_i;
-	int load_file_count;
-	GtkWidget * load_progressbar;
+  GList * file_list;
+  
+  gboolean stop_loading; /* a flag set to stop the loading of images in midprocess */
 };
 
+static void gnome_icon_selection_class_init (GnomeIconSelectionClass *klass);
+static void gnome_icon_selection_init       (GnomeIconSelection      *gis);
+
 static void gnome_icon_selection_destroy    (GtkObject               *object);
 static void gnome_icon_selection_finalize   (GObject                 *object);
 
@@ -66,24 +64,25 @@ static int sort_file_list		    (gconstpointer            a,
 					     gconstpointer            b);
 
 GNOME_CLASS_BOILERPLATE (GnomeIconSelection, gnome_icon_selection,
-			 GtkVBox, GTK_TYPE_VBOX)
+			 GtkVBox, gtk_vbox)
 
 static void
 gnome_icon_selection_class_init (GnomeIconSelectionClass *klass)
 {
-  GObjectClass *gobject_class;
   GtkObjectClass *object_class;
+  GObjectClass *gobject_class;
 
-  gobject_class = (GObjectClass*) klass;
   object_class = (GtkObjectClass*) klass;
+  gobject_class = (GObjectClass*) klass;
 
-  gobject_class->finalize = gnome_icon_selection_finalize;
   object_class->destroy = gnome_icon_selection_destroy;
+  gobject_class->finalize = gnome_icon_selection_finalize;
 }
 
 static void
-gnome_icon_selection_instance_init (GnomeIconSelection *gis)
+gnome_icon_selection_init (GnomeIconSelection *gis)
 {
+	GtkAdjustment *vadj;
 	GtkWidget *box;
 	GtkWidget *frame;
 	GtkWidget *sb;
@@ -106,24 +105,19 @@ gnome_icon_selection_instance_init (GnomeIconSelection *gis)
 	gtk_box_pack_start (GTK_BOX (box), frame, TRUE, TRUE, 0);
 	gtk_widget_show (frame);
 
-	sb = gtk_vscrollbar_new(NULL);
-	gtk_box_pack_end(GTK_BOX(box),sb,FALSE,FALSE,0);
-	gtk_widget_show(sb);
-
-	gis->_priv->gil = gnome_icon_list_new(ICON_SIZE+30,
-					      gtk_range_get_adjustment (GTK_RANGE (sb)),
-					      FALSE);
-	gtk_widget_set_size_request (gis->_priv->gil, 350, 300);
+	gis->_priv->gil = gnome_icon_list_new(ICON_SIZE+30, FALSE);
+	gtk_widget_set_usize(gis->_priv->gil,350,300);
 	gnome_icon_list_set_selection_mode(GNOME_ICON_LIST(gis->_priv->gil),
 					    GTK_SELECTION_SINGLE);
 
+	vadj = gtk_layout_get_vadjustment(GTK_LAYOUT(gis->_priv->gil));
+	sb = gtk_vscrollbar_new(vadj);
+	gtk_box_pack_end(GTK_BOX(box),sb,FALSE,FALSE,0);
+	gtk_widget_show(sb);
+
 	gtk_container_add (GTK_CONTAINER (frame), gis->_priv->gil);
 	gtk_widget_show(gis->_priv->gil);
 
-	g_signal_connect (gis->_priv->gil, "destroy",
-			  G_CALLBACK (gtk_widget_destroyed),
-			  &(gis->_priv->gil));
-
 	gis->_priv->file_list = NULL;
 }
 
@@ -141,7 +135,7 @@ gnome_icon_selection_new (void)
 {
   GnomeIconSelection * gis;
   
-  gis = g_object_new (GNOME_TYPE_ICON_SELECTION, NULL);
+  gis = gtk_type_new(gnome_icon_selection_get_type());
 
   return GTK_WIDGET (gis);
 }
@@ -151,44 +145,37 @@ gnome_icon_selection_destroy (GtkObject *object)
 {
 	GnomeIconSelection *gis;
 
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (GNOME_IS_ICON_SELECTION(object));
+	g_return_if_fail(object != NULL);
+	g_return_if_fail(GNOME_IS_ICON_SELECTION(object));
+
+	/* remember, destroy can be run multiple times! */
 	
-	gis = GNOME_ICON_SELECTION (object);
+	gis = GNOME_ICON_SELECTION(object);
 
-	if (gis->_priv->load_idle != 0)
-		g_source_remove (gis->_priv->load_idle);
-	gis->_priv->load_idle = 0;
+	/*clear our data if we have some*/
+	if(gis->_priv->file_list) {
+		g_list_foreach(gis->_priv->file_list,(GFunc)g_free,NULL);
+		g_list_free(gis->_priv->file_list);
+		gis->_priv->file_list = NULL;
+	}
 
-	if (gis->_priv->load_loop != NULL &&
-	    g_main_loop_is_running (gis->_priv->load_loop))
-		g_main_loop_quit (gis->_priv->load_loop);
-
-	GNOME_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
+	GNOME_CALL_PARENT_HANDLER (GTK_OBJECT_CLASS, destroy, (object));
 }
 
-
 static void
 gnome_icon_selection_finalize (GObject *object)
 {
 	GnomeIconSelection *gis;
 
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (GNOME_IS_ICON_SELECTION(object));
+	g_return_if_fail(object != NULL);
+	g_return_if_fail(GNOME_IS_ICON_SELECTION(object));
 	
-	gis = GNOME_ICON_SELECTION (object);
+	gis = GNOME_ICON_SELECTION(object);
 
-	/*clear our data if we have some*/
-	if (gis->_priv->file_list != NULL) {
-		g_list_foreach (gis->_priv->file_list, (GFunc)g_free, NULL);
-		g_list_free (gis->_priv->file_list);
-		gis->_priv->file_list = NULL;
-	}
-
-	g_free (gis->_priv);
+	g_free(gis->_priv);
 	gis->_priv = NULL;
 
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
+	GNOME_CALL_PARENT_HANDLER (G_OBJECT_CLASS, finalize, (object));
 }
 
 /**
@@ -206,8 +193,7 @@ gnome_icon_selection_add_defaults (GnomeIconSelection * gis)
 {
   gchar *pixmap_dir;
 
-  g_return_if_fail (gis != NULL);
-  g_return_if_fail (GNOME_IS_ICON_SELECTION (gis));
+  g_return_if_fail(gis != NULL);
 
   pixmap_dir = gnome_unconditional_datadir_file("pixmaps");
   
@@ -250,7 +236,7 @@ append_an_icon(GnomeIconSelection * gis, const gchar * path)
 		GdkPixbuf *scaled;
 	        scaled = gdk_pixbuf_scale_simple(pixbuf, w, h,
 						 GDK_INTERP_BILINEAR);
-		g_object_unref (G_OBJECT (pixbuf));
+		gdk_pixbuf_unref(pixbuf);
 		pixbuf = scaled;
 
 		/* sanity */
@@ -262,7 +248,7 @@ append_an_icon(GnomeIconSelection * gis, const gchar * path)
 	pos = gnome_icon_list_append_pixbuf(GNOME_ICON_LIST(gis->_priv->gil),
 					    pixbuf, path, base);
 	g_free(base);
-	g_object_unref (G_OBJECT (pixbuf)); /* I'm so glad that gdk-pixbuf has eliminated the former lameness of imlib! :) */
+	gdk_pixbuf_unref(pixbuf); /* I'm so glad that gdk-pixbuf has eliminated the former lameness of imlib! :) */
 }
 
 static int sort_file_list( gconstpointer a, gconstpointer b)
@@ -288,9 +274,8 @@ gnome_icon_selection_add_directory (GnomeIconSelection * gis,
   struct dirent * de;
   DIR * dp;
 
-  g_return_if_fail (gis != NULL);
-  g_return_if_fail (GNOME_IS_ICON_SELECTION (gis));
-  g_return_if_fail (dir != NULL);
+  g_return_if_fail(gis != NULL);
+  g_return_if_fail(dir != NULL);
 
   if ( ! g_file_test (dir, G_FILE_TEST_IS_DIR)) {
 	  g_warning(_("GnomeIconSelection: '%s' does not exist or is not "
@@ -307,26 +292,18 @@ gnome_icon_selection_add_directory (GnomeIconSelection * gis,
 
   while ( (de = readdir(dp)) != NULL ) {
     const char *mimetype;
-    char *uri;
-    GnomeVFSFileInfo *info;
-    gchar *full_path;
-    
+#ifdef GNOME_ENABLE_DEBUG
+    g_print("File: %s\n", de->d_name);
+#endif
     if ( *(de->d_name) == '.' ) continue; /* skip dotfiles */
 
-    full_path = g_build_filename (dir, de->d_name, NULL);
-    uri = g_filename_to_uri (full_path, "localhost", NULL);
-    info = gnome_vfs_file_info_new ();
-    g_free (full_path);
-    /* FIXME: OK to do synchronous I/O here? */
-    gnome_vfs_get_file_info (uri, info, GNOME_VFS_FILE_INFO_GET_MIME_TYPE | 
-			     GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
-    mimetype = info->mime_type;
-    g_free (uri);
-
+    mimetype = gnome_vfs_mime_type_from_name(de->d_name);
     if (mimetype != NULL &&
 	strncmp(mimetype, "image", strlen("image")) == 0 ) {
-      gchar * full_path = g_build_filename (dir, de->d_name, NULL);
-
+      gchar * full_path = g_concat_dir_and_file(dir, de->d_name);
+#ifdef GNOME_ENABLE_DEBUG
+      g_print("Full path: %s\n", full_path);
+#endif
       if (g_file_test (full_path, G_FILE_TEST_IS_REGULAR)) {
 	      /* Image filename, exists, regular file, go for it. */
 	      gis->_priv->file_list =
@@ -336,42 +313,15 @@ gnome_icon_selection_add_directory (GnomeIconSelection * gis,
       }
       g_free(full_path);
     }
-    gnome_vfs_file_info_unref (info);
   }
 
   closedir(dp);
 }
 
-static gboolean
-load_idle_func (gpointer data)
+static void
+set_flag(GtkWidget *w, int *flag)
 {
-	GnomeIconSelection * gis = data;
-	GList * list = gis->_priv->file_list;
-
-	if (list == NULL) {
-		if (gis->_priv->load_loop != NULL &&
-		    g_main_loop_is_running (gis->_priv->load_loop))
-			g_main_loop_quit (gis->_priv->load_loop);
-		gis->_priv->load_idle = 0;
-		return FALSE;
-	}
-
-	append_an_icon (gis, list->data);
-
-	g_free (list->data);
-	gis->_priv->file_list = g_list_remove_link (gis->_priv->file_list, list);
-	g_list_free_1 (list);
-
-	/* If icons were added while adding */
-	if (gis->_priv->load_i > gis->_priv->load_file_count)
-		gis->_priv->load_file_count = gis->_priv->load_i;
-
-	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (gis->_priv->load_progressbar),
-				       (double)gis->_priv->load_i / gis->_priv->load_file_count);
-
-	gis->_priv->load_i ++;
-
-	return TRUE;
+	*flag = TRUE;
 }
 
 /**
@@ -389,73 +339,99 @@ load_idle_func (gpointer data)
 void
 gnome_icon_selection_show_icons (GnomeIconSelection * gis)
 {
-	GtkWidget *label;
-	GtkWidget *progressbar;
+  GtkWidget *label;
+  GtkWidget *progressbar;
+  int file_count, i;
+  int local_dest;
+  int was_destroyed = FALSE;
 
-	g_return_if_fail (gis != NULL);
-	g_return_if_fail (GNOME_IS_ICON_SELECTION (gis));
+  g_return_if_fail(gis != NULL);
+  if(!gis->_priv->file_list) return;
 
-	/* Nothing to load */
-	if (gis->_priv->file_list == NULL)
-		return;
+  file_count = g_list_length(gis->_priv->file_list);
+  i = 0;
 
-	/* We're already loading! */
-	if (gis->_priv->load_loop != NULL)
-		return;
+  /* Locate previous progressbar/label,
+   * if previously called. */
+  progressbar = label = NULL;
+  progressbar = gtk_object_get_user_data(GTK_OBJECT(gis));
+  if (progressbar)
+         label = gtk_object_get_user_data(GTK_OBJECT(progressbar));
 
-	label = gtk_label_new (_("Loading Icons..."));
-	gtk_box_pack_start (GTK_BOX (gis->_priv->box),
-			    label, FALSE, FALSE, 0);
-	gtk_widget_show (label);
-	g_signal_connect (label, "destroy",
-			  G_CALLBACK (gtk_widget_destroyed),
-			  &label);
+  if (!label && !progressbar) {
+         label = gtk_label_new(_("Loading Icons..."));
+         gtk_box_pack_start(GTK_BOX(gis->_priv->box),label,FALSE,FALSE,0);
+         gtk_widget_show(label);
 
-	progressbar = gtk_progress_bar_new ();
-	gtk_box_pack_start (GTK_BOX (gis->_priv->box),
-			    progressbar, FALSE, FALSE, 0);
-	gtk_widget_show (progressbar);
-	g_signal_connect (progressbar, "destroy",
-			  G_CALLBACK (gtk_widget_destroyed),
-			  &progressbar);
+         progressbar = gtk_progress_bar_new();
+         gtk_box_pack_start(GTK_BOX(gis->_priv->box),progressbar,FALSE,FALSE,0);
+         gtk_widget_show(progressbar);
 
-	gnome_icon_list_freeze (GNOME_ICON_LIST (gis->_priv->gil));
+         /* attach label to progressbar, progressbar to gis 
+          * for recovery if show_icons() called again */
+         gtk_object_set_user_data(GTK_OBJECT(progressbar), label);
+         gtk_object_set_user_data(GTK_OBJECT(gis), progressbar);
+  } else {
+         if (!label && progressbar) g_assert_not_reached();
+         if (label && !progressbar) g_assert_not_reached();
+  }
+         
+  gnome_icon_list_freeze(GNOME_ICON_LIST(gis->_priv->gil));
 
-	/* Ref to avoid object disappearing from under us */
-	g_object_ref (G_OBJECT (gis));
+  /* this can be set with the stop_loading method to stop the
+     display in the middle */
+  gis->_priv->stop_loading = FALSE;
+  
+  /*bind destroy so that we can bail out of this function if the whole thing
+    was destroyed while doing the main_iteration*/
+  local_dest = gtk_signal_connect(GTK_OBJECT(gis),"destroy",
+				  GTK_SIGNAL_FUNC(set_flag),
+				  &was_destroyed);
 
-	gis->_priv->load_loop = g_main_loop_new (NULL, FALSE);
-	gis->_priv->load_i = 0;
-	gis->_priv->load_file_count = g_list_length (gis->_priv->file_list);
-	gis->_priv->load_progressbar = progressbar;
+  while (gis->_priv->file_list) {
+	  GList * list = gis->_priv->file_list;
+	  append_an_icon(gis, list->data);
+	  g_free(list->data);
+	  gis->_priv->file_list = g_list_remove_link(gis->_priv->file_list,list);
+	  g_list_free_1(list);
 
-	if (gis->_priv->load_idle != 0)
-		g_source_remove (gis->_priv->load_idle);
-	gis->_priv->load_idle = g_idle_add (load_idle_func, gis);
+	  gtk_progress_bar_update (GTK_PROGRESS_BAR (progressbar),
+				   (float)i / file_count);
+	  /* FIXME: this should be done either
+	   * 1) asynchroniously
+	   * 2) with a separate main loop and not the gtk one */
+	  while ( gtk_events_pending() ) {
+                  gtk_main_iteration();
 
-	GDK_THREADS_LEAVE ();  
-	g_main_loop_run (gis->_priv->load_loop);
-	GDK_THREADS_ENTER ();  
+                  /*if the gis was destroyed from underneath us ... bail out*/
+                  if(was_destroyed) 
+                          return;
+                  
+                  if(gis->_priv->stop_loading)
+                          goto out;
+	  }
+	  
+	  i++;
+  }
 
-	if (gis->_priv->load_idle != 0)
-		g_source_remove (gis->_priv->load_idle);
-	gis->_priv->load_idle = 0;
+ out:
+  
+  gtk_signal_disconnect(GTK_OBJECT(gis),local_dest);
 
-	if (gis->_priv->load_loop != NULL)
-		g_main_loop_unref (gis->_priv->load_loop);
-	gis->_priv->load_loop = NULL;
+  gnome_icon_list_thaw(GNOME_ICON_LIST(gis->_priv->gil));
 
-	gis->_priv->load_progressbar = NULL;
+  progressbar = label = NULL;
+  progressbar = gtk_object_get_user_data(GTK_OBJECT(gis));
+  if (progressbar)
+         label = gtk_object_get_user_data(GTK_OBJECT(progressbar));
+  if (progressbar)
+	  gtk_widget_destroy(progressbar);
+  if (label)
+	  gtk_widget_destroy(label);
 
-	if (gis->_priv->gil != NULL)
-		gnome_icon_list_thaw (GNOME_ICON_LIST (gis->_priv->gil));
-
-	if (progressbar != NULL)
-		gtk_widget_destroy (progressbar);
-	if (label != NULL)
-		gtk_widget_destroy (label);
-
-	g_object_unref (G_OBJECT (gis));
+  /* cleanse gis of evil progressbar/label ptrs */
+  /* also let previous calls to show_icons() know that rendering is done. */
+  gtk_object_set_user_data(GTK_OBJECT(gis), NULL);
 }
 
 /**
@@ -473,14 +449,12 @@ gnome_icon_selection_show_icons (GnomeIconSelection * gis)
  * Returns:
  **/
 void
-gnome_icon_selection_stop_loading (GnomeIconSelection * gis)
+gnome_icon_selection_stop_loading(GnomeIconSelection * gis)
 {
-	g_return_if_fail (gis != NULL);
-	g_return_if_fail (GNOME_IS_ICON_SELECTION (gis));
+	g_return_if_fail(gis != NULL);
+	g_return_if_fail(GNOME_IS_ICON_SELECTION(gis));
 	
-	if (gis->_priv->load_loop != NULL &&
-	    g_main_loop_is_running (gis->_priv->load_loop))
-		g_main_loop_quit (gis->_priv->load_loop);
+	gis->_priv->stop_loading = TRUE;
 }
 
 /**
@@ -530,8 +504,7 @@ gnome_icon_selection_get_icon (GnomeIconSelection * gis,
 {
 	GList * sel;
 
-	g_return_val_if_fail (gis != NULL, NULL);
-	g_return_val_if_fail (GNOME_IS_ICON_SELECTION (gis), NULL);
+	g_return_val_if_fail(gis != NULL, NULL);
 
 	sel = gnome_icon_list_get_selection(GNOME_ICON_LIST(gis->_priv->gil));
 	if (sel != NULL) {
@@ -565,9 +538,8 @@ gnome_icon_selection_select_icon (GnomeIconSelection * gis,
 	gint pos;
 	gint icons;
 
-	g_return_if_fail (gis != NULL);
-	g_return_if_fail (GNOME_IS_ICON_SELECTION (gis));
-	g_return_if_fail (filename != NULL);
+	g_return_if_fail(gis != NULL);
+	g_return_if_fail(filename != NULL);
 
 	gil = GNOME_ICON_LIST(gis->_priv->gil);
 	icons = gnome_icon_list_get_num_icons(gil);
