@@ -22,7 +22,7 @@
 */
 
 #include <config.h>
-#include <libgnome/gnome-macros.h>
+#include "gnome-macros.h"
 
 #ifndef GNOME_DISABLE_DEPRECATED_SOURCE
 
@@ -37,7 +37,7 @@
 #include "gnome-mdi.h"
 
 static void       gnome_mdi_child_class_init       (GnomeMDIChildClass *klass);
-static void       gnome_mdi_child_instance_init    (GnomeMDIChild *);
+static void       gnome_mdi_child_init             (GnomeMDIChild *);
 static void       gnome_mdi_child_destroy          (GtkObject *);
 static void       gnome_mdi_child_finalize         (GObject *);
 
@@ -45,11 +45,11 @@ static GtkWidget *gnome_mdi_child_set_label        (GnomeMDIChild *, GtkWidget *
 static GtkWidget *gnome_mdi_child_create_view      (GnomeMDIChild *);
 
 /* declare the functions from gnome-mdi.c that we need but are not public */
-void _gnome_mdi_child_list_menu_remove_item (GnomeMDI *, GnomeMDIChild *);
-void _gnome_mdi_child_list_menu_add_item    (GnomeMDI *, GnomeMDIChild *);
+void child_list_menu_remove_item (GnomeMDI *, GnomeMDIChild *);
+void child_list_menu_add_item    (GnomeMDI *, GnomeMDIChild *);
 
 GNOME_CLASS_BOILERPLATE (GnomeMDIChild, gnome_mdi_child,
-						 GtkObject, GTK_TYPE_OBJECT)
+			 GtkObject, gtk_object)
 
 
 static void gnome_mdi_child_class_init (GnomeMDIChildClass *klass)
@@ -69,7 +69,7 @@ static void gnome_mdi_child_class_init (GnomeMDIChildClass *klass)
 	klass->set_label = gnome_mdi_child_set_label;
 }
 
-static void gnome_mdi_child_instance_init (GnomeMDIChild *mdi_child)
+static void gnome_mdi_child_init (GnomeMDIChild *mdi_child)
 {
 	mdi_child->name = NULL;
 	mdi_child->parent = NULL;
@@ -194,7 +194,7 @@ void gnome_mdi_child_set_name(GnomeMDIChild *mdi_child, const gchar *name)
 	gchar *old_name = mdi_child->name;
 
 	if(mdi_child->parent)
-		_gnome_mdi_child_list_menu_remove_item(GNOME_MDI(mdi_child->parent), mdi_child);
+		child_list_menu_remove_item(GNOME_MDI(mdi_child->parent), mdi_child);
 
 	mdi_child->name = (gchar *)g_strdup(name);
 
@@ -202,7 +202,7 @@ void gnome_mdi_child_set_name(GnomeMDIChild *mdi_child, const gchar *name)
 		g_free(old_name);
 
 	if(mdi_child->parent) {
-		_gnome_mdi_child_list_menu_add_item(GNOME_MDI(mdi_child->parent), mdi_child);
+		child_list_menu_add_item(GNOME_MDI(mdi_child->parent), mdi_child);
 		gnome_mdi_update_child(GNOME_MDI(mdi_child->parent), mdi_child);
 	}
 }
