@@ -21,7 +21,6 @@
 */
 
 #include <config.h>
-#include <libgnome/gnome-macros.h>
 
 #ifndef GNOME_DISABLE_DEPRECATED_SOURCE
 
@@ -47,27 +46,86 @@ struct _GnomeMessageBoxPrivate {
 	int dummy;
 };
 
-GNOME_CLASS_BOILERPLATE (GnomeMessageBox, gnome_message_box,
-			 GnomeDialog, GNOME_TYPE_DIALOG)
+static void gnome_message_box_class_init (GnomeMessageBoxClass *klass);
+static void gnome_message_box_init       (GnomeMessageBox      *messagebox);
+static void gnome_message_box_destroy    (GtkObject            *object);
+static void gnome_message_box_finalize   (GObject              *object);
+static void gnome_message_box_construct  (GnomeMessageBox      *messagebox,
+					  const gchar          *message,
+					  const gchar          *message_box_type,
+					  const gchar         **buttons);
+
+static GnomeDialogClass *parent_class;
+
+guint
+gnome_message_box_get_type (void)
+{
+	static guint message_box_type = 0;
+
+	if (!message_box_type)
+	{
+		GtkTypeInfo message_box_info =
+		{
+			"GnomeMessageBox",
+			sizeof (GnomeMessageBox),
+			sizeof (GnomeMessageBoxClass),
+			(GtkClassInitFunc) gnome_message_box_class_init,
+			(GtkObjectInitFunc) gnome_message_box_init,
+			NULL,
+			NULL,
+			NULL
+		};
+
+		message_box_type = gtk_type_unique (gnome_dialog_get_type (), &message_box_info);
+	}
+
+	return message_box_type;
+}
 
 static void
 gnome_message_box_class_init (GnomeMessageBoxClass *klass)
 {
-	/*
 	GtkObjectClass *object_class;
+	GObjectClass *gobject_class;
 
 	object_class = (GtkObjectClass *)klass;
-	*/
+	gobject_class = (GObjectClass *)klass;
+	parent_class = gtk_type_class (gnome_dialog_get_type ());
+
+	object_class->destroy = gnome_message_box_destroy;
+	gobject_class->finalize = gnome_message_box_finalize;
 }
 
 static void
-gnome_message_box_instance_init (GnomeMessageBox *message_box)
+gnome_message_box_init (GnomeMessageBox *message_box)
 {
 	/*
  	message_box->_priv = g_new0(GnomeMessageBoxPrivate, 1); 
 	*/
 }
 
+static void
+gnome_message_box_destroy(GtkObject *object)
+{
+	/* remember, destroy can be run multiple times! */
+	if (GTK_OBJECT_CLASS (parent_class)->destroy)
+		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+}
+
+static void
+gnome_message_box_finalize(GObject *object)
+{
+	/*
+	GnomeMessageBox *mbox = GNOME_MESSAGE_BOX(object);
+
+	g_free(mbox->_priv);
+	mbox->_priv = NULL;
+	*/
+
+	if (G_OBJECT_CLASS (parent_class)->finalize)
+		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
+}
+
 /**
  * gnome_message_box_construct:
  * @messagebox: The message box to construct
@@ -80,7 +138,7 @@ gnome_message_box_instance_init (GnomeMessageBox *message_box)
  *
  * Returns:
  */
-void
+static void
 gnome_message_box_construct (GnomeMessageBox       *messagebox,
 			     const gchar           *message,
 			     const gchar           *message_box_type,
@@ -94,6 +152,7 @@ gnome_message_box_construct (GnomeMessageBox       *messagebox,
 	GtkStyle *style;
         const gchar* title_prefix = NULL;
         const gchar* appname;
+	gint i = 0;
 
 	g_return_if_fail (messagebox != NULL);
 	g_return_if_fail (GNOME_IS_MESSAGE_BOX (messagebox));
@@ -108,24 +167,38 @@ gnome_message_box_construct (GnomeMessageBox       *messagebox,
 	if (strcmp(GNOME_MESSAGE_BOX_INFO, message_box_type) == 0)
 	{
                 title_prefix = _("Information");
-		pixmap = gtk_image_new_from_stock (GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_DIALOG);
+		s = GNOMEUIPIXMAPDIR "/gnome-info.png";
+		if (s) {
+                        pixmap = gtk_image_new_from_file (s);
+                        g_free(s);
+                }
 	}
 	else if (strcmp(GNOME_MESSAGE_BOX_WARNING, message_box_type) == 0)
 	{
                 title_prefix = _("Warning");
-		pixmap = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING, GTK_ICON_SIZE_DIALOG);
-		
+		s = GNOMEUIPIXMAPDIR "/gnome-warning.png";
+		if (s) {
+                        pixmap = gtk_image_new_from_file (s);
+                        g_free(s);
+                }
 	}
 	else if (strcmp(GNOME_MESSAGE_BOX_ERROR, message_box_type) == 0)
 	{
                 title_prefix = _("Error");
-		pixmap = gtk_image_new_from_stock (GTK_STOCK_DIALOG_ERROR, GTK_ICON_SIZE_DIALOG);
-
+		s = GNOMEUIPIXMAPDIR "/gnome-error.png";
+		if (s) {
+                        pixmap = gtk_image_new_from_file (s);
+                        g_free(s);
+                }
 	}
 	else if (strcmp(GNOME_MESSAGE_BOX_QUESTION, message_box_type) == 0)
 	{
                 title_prefix = _("Question");
-		pixmap = gtk_image_new_from_stock (GTK_STOCK_DIALOG_QUESTION, GTK_ICON_SIZE_DIALOG);
+		s = GNOMEUIPIXMAPDIR "/gnome-question.png";
+		if (s) {
+                        pixmap = gtk_image_new_from_file (s);
+                        g_free(s);
+                }
 	}
 	else
 	{
@@ -138,15 +211,27 @@ gnome_message_box_construct (GnomeMessageBox       *messagebox,
         if (appname) {
                 s = g_strdup_printf("%s (%s)", title_prefix, appname);
         }
-
-	gnome_dialog_constructv (GNOME_DIALOG (messagebox), s ? s : title_prefix, buttons);
-	g_free (s);
+        if (s) {
+                gtk_window_set_title(GTK_WINDOW(messagebox), s);
+                g_free(s);
+        } else {
+                gtk_window_set_title(GTK_WINDOW(messagebox), title_prefix);
+        }
 
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX(GNOME_DIALOG(messagebox)->vbox),
 			    hbox, TRUE, TRUE, 10);
 	gtk_widget_show (hbox);
 
+	if (pixmap == NULL) {
+        	if (pixmap) gtk_widget_destroy(pixmap);
+		s = GNOMEUIPIXMAPDIR "/gnome-default-dlg.png";
+         	if (s) {
+			pixmap = gtk_image_new_from_file (s);
+                        g_free(s);
+                } else
+			pixmap = NULL;
+	}
 	if (pixmap) {
 		gtk_box_pack_start (GTK_BOX(hbox), 
 				    pixmap, FALSE, TRUE, 0);
@@ -163,11 +248,23 @@ gnome_message_box_construct (GnomeMessageBox       *messagebox,
 	/* Add some extra space on the right to balance the pixmap */
 	if (pixmap) {
 		alignment = gtk_alignment_new (0., 0., 0., 0.);
-		gtk_widget_set_size_request (alignment, GNOME_PAD, -1);
+		gtk_widget_set_usize (alignment, GNOME_PAD, -1);
 		gtk_widget_show (alignment);
 		
 		gtk_box_pack_start (GTK_BOX (hbox), alignment, FALSE, FALSE, 0);
 	}
+
+	if (buttons) {
+		while (buttons[i]) {
+			gnome_dialog_append_button (GNOME_DIALOG (messagebox), 
+						    buttons[i]);
+			i++;
+		};
+	}
+
+	if(GNOME_DIALOG(messagebox)->buttons)
+		gtk_widget_grab_focus(
+			g_list_last (GNOME_DIALOG (messagebox)->buttons)->data);
 	
 	gnome_dialog_set_close (GNOME_DIALOG (messagebox),
 				TRUE );
@@ -197,7 +294,7 @@ gnome_message_box_new (const gchar           *message,
         
 	va_start (ap, message_box_type);
 	
-	message_box = g_object_new (GNOME_TYPE_MESSAGE_BOX, NULL);
+	message_box = gtk_type_new (gnome_message_box_get_type ());
 
 	gnome_message_box_construct (message_box, message,
 				     message_box_type, NULL);
@@ -218,10 +315,8 @@ gnome_message_box_new (const gchar           *message,
 	
 	va_end (ap);
 
-	if (GNOME_DIALOG (message_box)->buttons) {
-		gtk_widget_grab_focus(
-			g_list_last (GNOME_DIALOG (message_box)->buttons)->data);
-	}
+	gtk_widget_grab_focus(
+		g_list_last (GNOME_DIALOG (message_box)->buttons)->data);
 
 	return GTK_WIDGET (message_box);
 }
@@ -249,7 +344,7 @@ gnome_message_box_newv (const gchar           *message,
 	g_return_val_if_fail (message != NULL, NULL);
 	g_return_val_if_fail (message_box_type != NULL, NULL);
 
-	message_box = g_object_new (GNOME_TYPE_MESSAGE_BOX, NULL);
+	message_box = gtk_type_new (gnome_message_box_get_type ());
 
 	gnome_message_box_construct (message_box, message,
 				     message_box_type, buttons);
