@@ -31,7 +31,7 @@
  * Author: George Lebl <jirka@5z.com>
  */
 #include <config.h>
-#include <libgnome/gnome-macros.h>
+#include "gnome-macros.h"
 
 #include <sys/stat.h>
 #include <unistd.h>
@@ -46,7 +46,6 @@
 #include <gtk/gtksignal.h>
 #include <gtk/gtkpixmap.h>
 #include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtktreeview.h>
 
 #include "gnome-i18nP.h"
 
@@ -55,15 +54,12 @@
 #include "gnome-pixmap-entry.h"
 #include "gnome-pixmap.h"
 
-#include <libgnomevfs/gnome-vfs-uri.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
-
-#include "libgnomeui-access.h"
+#include <libgnomevfs/gnome-vfs-mime.h>
 
 struct _GnomePixmapEntryPrivate {
 	GtkWidget *preview;
 	GtkWidget *preview_sw;
-
+	
 	gchar *last_preview;
 
 	guint32 do_preview : 1; /*put a preview frame with the pixmap next to
@@ -71,30 +67,40 @@ struct _GnomePixmapEntryPrivate {
 };
 
 
+static void gnome_pixmap_entry_class_init (GnomePixmapEntryClass *class);
+static void gnome_pixmap_entry_init       (GnomePixmapEntry      *pentry);
 static void drag_data_get		  (GtkWidget          *widget,
 					   GdkDragContext     *context,
 					   GtkSelectionData   *selection_data,
 					   guint               info,
 					   guint               time,
 					   GnomePixmapEntry   *pentry);
+static void drag_data_received		  (GtkWidget        *widget,
+					   GdkDragContext   *context,
+					   gint              x,
+					   gint              y,
+					   GtkSelectionData *selection_data,
+					   guint             info,
+					   guint32           time,
+					   GnomePixmapEntry *pentry);
 static void pentry_destroy		  (GtkObject *object);
 static void pentry_finalize		  (GObject *object);
-
-static void pentry_set_property (GObject *object, guint param_id,
-				 const GValue *value, GParamSpec *pspec);
-static void pentry_get_property (GObject *object, guint param_id,
-				 GValue *value, GParamSpec *pspec);
+static void pentry_set_arg                (GtkObject *object,
+					   GtkArg *arg,
+					   guint arg_id);
+static void pentry_get_arg                (GtkObject *object,
+					   GtkArg *arg,
+					   guint arg_id);
 
 static GtkTargetEntry drop_types[] = { { "text/uri-list", 0, 0 } };
 
-/* Property IDs */
 enum {
-	PROP_0,
-	PROP_DO_PREVIEW
+	ARG_0,
+	ARG_DO_PREVIEW
 };
 
 GNOME_CLASS_BOILERPLATE (GnomePixmapEntry, gnome_pixmap_entry,
-			 GnomeFileEntry, GNOME_TYPE_FILE_ENTRY)
+			 GnomeFileEntry, gnome_file_entry)
 
 static void
 gnome_pixmap_entry_class_init (GnomePixmapEntryClass *class)
@@ -102,60 +108,51 @@ gnome_pixmap_entry_class_init (GnomePixmapEntryClass *class)
 	GtkObjectClass *object_class = GTK_OBJECT_CLASS(class);
 	GObjectClass *gobject_class = G_OBJECT_CLASS(class);
 
-	gobject_class->finalize = pentry_finalize;
-	gobject_class->set_property = pentry_set_property;
-	gobject_class->get_property = pentry_get_property;
-
-	g_object_class_install_property (gobject_class,
-					 PROP_DO_PREVIEW,
-					 g_param_spec_boolean (
-						 "do_preview",
-						 _("Do Preview"),
-						 _("Whether the pixmap entry should have a preview."),
-						 FALSE,
-						 G_PARAM_READWRITE));
+	gtk_object_add_arg_type("GnomePixmapEntry::do_preview",
+				GTK_TYPE_BOOL,
+				GTK_ARG_READWRITE,
+				ARG_DO_PREVIEW);
 
 	object_class->destroy = pentry_destroy;
+	gobject_class->finalize = pentry_finalize;
+	object_class->get_arg = pentry_get_arg;
+	object_class->set_arg = pentry_set_arg;
+
 }
 
-/* set_property handler for the pixmap entry */
 static void
-pentry_set_property (GObject *object, guint param_id,
-		     const GValue *value, GParamSpec *pspec)
+pentry_set_arg (GtkObject *object,
+		GtkArg *arg,
+		guint arg_id)
 {
-	GnomePixmapEntry *pentry;
+	GnomePixmapEntry *self;
 
-	pentry = GNOME_PIXMAP_ENTRY (object);
+	self = GNOME_PIXMAP_ENTRY (object);
 
-	switch (param_id) {
-	case PROP_DO_PREVIEW:
-		gnome_pixmap_entry_set_preview(pentry, g_value_get_boolean (value));
+	switch (arg_id) {
+	case ARG_DO_PREVIEW:
+		gnome_pixmap_entry_set_preview(self, GTK_VALUE_BOOL(*arg));
 		break;
 
 	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
 	}
 }
 
-/* get_property handler for the pixmap entry */
 static void
-pentry_get_property (GObject *object, guint param_id,
-		     GValue *value, GParamSpec *pspec)
+pentry_get_arg (GtkObject *object,
+		GtkArg *arg,
+		guint arg_id)
 {
-	GnomePixmapEntry *pentry;
-	GnomePixmapEntryPrivate *priv;
+	GnomePixmapEntry *self;
 
-	pentry = GNOME_PIXMAP_ENTRY (object);
-	priv = pentry->_priv;
+	self = GNOME_PIXMAP_ENTRY (object);
 
-	switch (param_id) {
-	case PROP_DO_PREVIEW:
-		g_value_set_boolean (value, priv->do_preview);
+	switch (arg_id) {
+	case ARG_DO_PREVIEW:
+		GTK_VALUE_BOOL(*arg) = self->_priv->do_preview;
 		break;
-
 	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
 	}
 }
@@ -171,7 +168,7 @@ refresh_preview(GnomePixmapEntry *pentry)
 
 	if( ! pentry->_priv->preview)
 		return;
-
+	
 	t = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(pentry), FALSE);
 
 	if(pentry->_priv->last_preview && t
@@ -201,23 +198,19 @@ refresh_preview(GnomePixmapEntry *pentry)
         } else {
 		gtk_widget_destroy(pentry->_priv->preview->parent);
 		pentry->_priv->preview = gtk_image_new_from_pixbuf (pixbuf);
-		_add_atk_name_desc (pentry->_priv->preview,
-				    _("Image Preview"),
-				    _("A preview of the image currently specified"));
-
 		gtk_widget_show(pentry->_priv->preview);
 		gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(pentry->_priv->preview_sw),
 						      pentry->_priv->preview);
 		if(!GTK_WIDGET_NO_WINDOW(pentry->_priv->preview)) {
-			g_signal_connect (G_OBJECT (pentry->_priv->preview), "drag_data_get",
-					  G_CALLBACK (drag_data_get), pentry);
+			gtk_signal_connect (GTK_OBJECT (pentry->_priv->preview), "drag_data_get",
+					    GTK_SIGNAL_FUNC (drag_data_get),pentry);
 			gtk_drag_source_set (pentry->_priv->preview,
 					     GDK_BUTTON1_MASK|GDK_BUTTON3_MASK,
 					     drop_types, 1,
 					     GDK_ACTION_COPY);
 		}
-		g_signal_connect (G_OBJECT (pentry->_priv->preview->parent), "drag_data_get",
-				  G_CALLBACK (drag_data_get), pentry);
+		gtk_signal_connect (GTK_OBJECT (pentry->_priv->preview->parent), "drag_data_get",
+				    GTK_SIGNAL_FUNC (drag_data_get),pentry);
 		gtk_drag_source_set (pentry->_priv->preview->parent,
 				     GDK_BUTTON1_MASK|GDK_BUTTON3_MASK,
 				     drop_types, 1,
@@ -225,7 +218,7 @@ refresh_preview(GnomePixmapEntry *pentry)
 	}
 	g_free(pentry->_priv->last_preview);
 	pentry->_priv->last_preview = t;
-        g_object_unref (G_OBJECT (pixbuf));
+        gdk_pixbuf_unref(pixbuf);
 }
 
 static int change_timeout;
@@ -268,7 +261,7 @@ entry_changed(GtkWidget *w, GnomePixmapEntry *pentry)
 }
 
 static void
-setup_preview(GtkTreeSelection *selection, GtkWidget *widget)
+setup_preview(GtkWidget *widget)
 {
 	const char *p;
 	GList *l;
@@ -278,10 +271,11 @@ setup_preview(GtkTreeSelection *selection, GtkWidget *widget)
 	GtkWidget *frame;
 	GtkFileSelection *fs;
 
+	g_return_if_fail (widget != NULL);
 	g_return_if_fail (GTK_IS_WIDGET (widget));
 
-	frame = g_object_get_data (G_OBJECT (widget), "frame");
-	fs = g_object_get_data (G_OBJECT (frame), "fs");
+	frame = gtk_object_get_data(GTK_OBJECT(widget),"frame");
+	fs = gtk_object_get_data(GTK_OBJECT(frame),"fs");
 
 	if((l = gtk_container_get_children(GTK_CONTAINER(frame))) != NULL) {
 		pp = l->data;
@@ -311,9 +305,9 @@ setup_preview(GtkTreeSelection *selection, GtkWidget *widget)
 	}
 	scaled = gdk_pixbuf_scale_simple
 		(pixbuf, w, h, GDK_INTERP_BILINEAR);
-        g_object_unref (G_OBJECT (pixbuf));
+        gdk_pixbuf_unref (pixbuf);
 	pp = gtk_image_new_from_pixbuf (scaled);
-        g_object_unref (G_OBJECT (scaled));
+        gdk_pixbuf_unref (scaled);
 
 	gtk_widget_show(pp);
 
@@ -322,7 +316,7 @@ setup_preview(GtkTreeSelection *selection, GtkWidget *widget)
 }
 
 static void
-pentry_destroy (GtkObject *object)
+pentry_destroy(GtkObject *object)
 {
 	GnomePixmapEntry *pentry;
 
@@ -331,30 +325,34 @@ pentry_destroy (GtkObject *object)
 
 	/* remember, destroy can be run multiple times! */
 
-	pentry = GNOME_PIXMAP_ENTRY (object);
+	pentry = GNOME_PIXMAP_ENTRY(object);
 
-	changed_pentries = g_slist_remove (changed_pentries, pentry);
+	g_free(pentry->_priv->last_preview);
+	pentry->_priv->last_preview = NULL;
+
+	changed_pentries = g_slist_remove(changed_pentries, pentry);
+
+	if(GTK_OBJECT_CLASS(parent_class)->destroy)
+		(* GTK_OBJECT_CLASS(parent_class)->destroy)(object);
 
-	GNOME_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
 static void
-pentry_finalize (GObject *object)
+pentry_finalize(GObject *object)
 {
 	GnomePixmapEntry *pentry;
 
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (GNOME_IS_PIXMAP_ENTRY (object));
 
-	pentry = GNOME_PIXMAP_ENTRY (object);
+	pentry = GNOME_PIXMAP_ENTRY(object);
 
-	g_free (pentry->_priv->last_preview);
-	pentry->_priv->last_preview = NULL;
-
-	g_free (pentry->_priv);
+	g_free(pentry->_priv);
 	pentry->_priv = NULL;
 
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
+	if(G_OBJECT_CLASS(parent_class)->finalize)
+		(* G_OBJECT_CLASS(parent_class)->finalize)(object);
+
 }
 
 static void
@@ -372,13 +370,8 @@ browse_clicked(GnomeFileEntry *fentry, GnomePixmapEntry *pentry)
 
 	if(!fentry->fsw)
 		return;
-
 	fs = GTK_FILE_SELECTION(fentry->fsw);
 
-	/* we already got this */
-	if (g_object_get_data (G_OBJECT (fs->file_list), "frame") != NULL)
-		return;
-
 	hbox = fs->file_list;
 	do {
 		hbox = hbox->parent;
@@ -392,15 +385,62 @@ browse_clicked(GnomeFileEntry *fentry, GnomePixmapEntry *pentry)
 	w = gtk_frame_new(_("Preview"));
 	gtk_widget_show(w);
 	gtk_box_pack_end(GTK_BOX(hbox),w,FALSE,FALSE,0);
-	gtk_widget_set_size_request (w, 110, 110);
-	g_object_set_data (G_OBJECT (w), "fs", fs);
+	gtk_widget_set_usize(w,110,110);
+	gtk_object_set_data(GTK_OBJECT(w),"fs",fs);
 
-	g_object_set_data (G_OBJECT (fs->file_list), "frame", w);
-	g_signal_connect (G_OBJECT (gtk_tree_view_get_selection (GTK_TREE_VIEW (fs->file_list))), "changed",
-			  G_CALLBACK (setup_preview), fs->file_list);
-	g_object_set_data (G_OBJECT (fs->selection_entry), "frame", w);
-	g_signal_connect (fs->selection_entry, "changed",
-			  G_CALLBACK (setup_preview), fs->selection_entry);
+	gtk_object_set_data(GTK_OBJECT(fs->file_list),"frame",w);
+	gtk_signal_connect(GTK_OBJECT(fs->file_list),"select_row",
+			   GTK_SIGNAL_FUNC(setup_preview),NULL);
+	gtk_object_set_data(GTK_OBJECT(fs->selection_entry),"frame",w);
+	gtk_signal_connect(GTK_OBJECT(fs->selection_entry),"changed",
+			   GTK_SIGNAL_FUNC(setup_preview),NULL);
+}
+
+static void
+drag_data_received (GtkWidget        *widget,
+		    GdkDragContext   *context,
+		    gint              x,
+		    gint              y,
+		    GtkSelectionData *selection_data,
+		    guint             info,
+		    guint32           time,
+		    GnomePixmapEntry *pentry)
+{
+	GList *uris, *li;
+	GnomeVFSURI *uri = NULL;
+	GtkWidget *entry;
+	
+	g_return_if_fail (pentry != NULL);
+	g_return_if_fail (GNOME_IS_PIXMAP_ENTRY (pentry));
+
+	entry = gnome_pixmap_entry_gtk_entry (pentry);
+
+	/*here we extract the filenames from the URI-list we recieved*/
+	uris = gnome_vfs_uri_list_parse (selection_data->data);
+
+	/* FIXME: Support multiple files */
+	for (li = uris; li != NULL; li = li->next) {
+		uri = li->data;
+		/* FIXME: Support non-local files */
+		if (gnome_vfs_uri_is_local (uri)) {
+			break;
+		}
+		uri = NULL;
+	}
+
+	/*if there's isn't a file*/
+	if (uri == NULL) {
+		gtk_drag_finish (context, FALSE, FALSE, time);
+		/*free the list of files we got*/
+		gnome_vfs_uri_list_free (uris);
+		return;
+	}
+
+	gtk_entry_set_text (GTK_ENTRY (entry),
+			    gnome_vfs_uri_get_path (uri));
+
+	/*free the list of files we got*/
+	gnome_vfs_uri_list_free (uris);
 }
 
 static void
@@ -437,14 +477,21 @@ turn_on_previewbox(GnomePixmapEntry *pentry)
 {
 	pentry->_priv->preview_sw = gtk_scrolled_window_new(NULL,NULL);
 	gtk_widget_ref(pentry->_priv->preview_sw);
+	gtk_drag_dest_set (GTK_WIDGET (pentry->_priv->preview_sw),
+			   GTK_DEST_DEFAULT_MOTION |
+			   GTK_DEST_DEFAULT_HIGHLIGHT |
+			   GTK_DEST_DEFAULT_DROP,
+			   drop_types, 1, GDK_ACTION_COPY);
+	gtk_signal_connect (GTK_OBJECT (pentry->_priv->preview_sw), "drag_data_received",
+			    GTK_SIGNAL_FUNC (drag_data_received),pentry);
 	/*for some reason we can't do this*/
-	/*g_signal_connect (G_OBJECT (pentry->_priv->preview_sw), "drag_data_get",
-			    G_CALLBACK (drag_data_get), pentry);*/
+	/*gtk_signal_connect (GTK_OBJECT (pentry->_priv->preview_sw), "drag_data_get",
+			    GTK_SIGNAL_FUNC (drag_data_get),pentry);*/
 	gtk_box_pack_start (GTK_BOX (pentry), pentry->_priv->preview_sw, TRUE, TRUE, 0);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(pentry->_priv->preview_sw),
 				       GTK_POLICY_AUTOMATIC,
 				       GTK_POLICY_AUTOMATIC);
-	gtk_widget_set_size_request (pentry->_priv->preview_sw, 100, 100);
+	gtk_widget_set_usize(pentry->_priv->preview_sw,100,100);
 	gtk_widget_show (pentry->_priv->preview_sw);
 
 	pentry->_priv->preview = gtk_label_new(_("No Image"));
@@ -458,7 +505,7 @@ turn_on_previewbox(GnomePixmapEntry *pentry)
 }
 
 static void
-gnome_pixmap_entry_instance_init (GnomePixmapEntry *pentry)
+gnome_pixmap_entry_init (GnomePixmapEntry *pentry)
 {
 	GtkWidget *w;
 	char *p;
@@ -467,16 +514,16 @@ gnome_pixmap_entry_instance_init (GnomePixmapEntry *pentry)
 
 	pentry->_priv = g_new0(GnomePixmapEntryPrivate, 1);
 
-	pentry->_priv->do_preview = 0;
+	pentry->_priv->do_preview = 1;
 
 	pentry->_priv->last_preview = NULL;
 
 	pentry->_priv->preview = NULL;
 	pentry->_priv->preview_sw = NULL;
 
-	g_signal_connect_after (pentry, "browse_clicked",
-				G_CALLBACK (browse_clicked),
-				pentry);
+	gtk_signal_connect_after(GTK_OBJECT(pentry),"browse_clicked",
+				 GTK_SIGNAL_FUNC(browse_clicked),
+				 pentry);
 
 	p = gnome_program_locate_file (NULL /* program */,
 				       GNOME_FILE_DOMAIN_PIXMAP,
@@ -487,8 +534,9 @@ gnome_pixmap_entry_instance_init (GnomePixmapEntry *pentry)
 	g_free(p);
 
 	w = gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(pentry));
-	g_signal_connect (G_OBJECT (w), "changed",
-			  G_CALLBACK (entry_changed), pentry);
+	gtk_signal_connect(GTK_OBJECT(w),"changed",
+			   GTK_SIGNAL_FUNC(entry_changed),
+			   pentry);
 
 	/*just in case there is a default that is an image*/
 	refresh_preview(pentry);
@@ -538,7 +586,7 @@ gnome_pixmap_entry_new (const gchar *history_id, const gchar *browse_dialog_titl
 {
 	GnomePixmapEntry *pentry;
 
-	pentry = g_object_new (GNOME_TYPE_PIXMAP_ENTRY, NULL);
+	pentry = gtk_type_new (gnome_pixmap_entry_get_type ());
 
 	gnome_pixmap_entry_construct (pentry, history_id, browse_dialog_title, do_preview);
 	return GTK_WIDGET (pentry);
@@ -678,7 +726,7 @@ gnome_pixmap_entry_set_pixmap_subdir(GnomePixmapEntry *pentry,
  * the entry.  If the preview is on, we also load the files and check
  * for them being real images.  If it is off, we don't check files
  * to be real image files.
- *
+ * 
  * Returns:
  **/
 void
@@ -730,8 +778,8 @@ gnome_pixmap_entry_set_preview_size(GnomePixmapEntry *pentry,
 	g_return_if_fail (preview_w>=0 && preview_h>=0);
 
 	if(pentry->_priv->preview_sw)
-		gtk_widget_set_size_request (pentry->_priv->preview_sw,
-					     preview_w, preview_h);
+		gtk_widget_set_usize(pentry->_priv->preview_sw,
+				     preview_w,preview_h);
 }
 
 /* Ensures that a pixmap entry is not in the waiting list for a preview update.  */
@@ -765,7 +813,7 @@ ensure_update (GnomePixmapEntry *pentry)
  * successfully loaded if preview is disabled.  If the preview is
  * disabled, the file is only checked if it exists or not.
  *
- * Returns: Newly allocated string containing path, or %NULL on error.
+ * Returns: Newly allocated string containing path, or %NULL on error. 
  **/
 char *
 gnome_pixmap_entry_get_filename(GnomePixmapEntry *pentry)
