@@ -67,18 +67,14 @@ struct _GnomePixmapEntry {
 
 struct _GnomePixmapEntryClass {
 	GnomeFileEntryClass parent_class;
-
-	/* Padding for possible expansion */
-	gpointer padding1;
-	gpointer padding2;
 };
 
 
-GType      gnome_pixmap_entry_get_type    (void) G_GNUC_CONST;
+guint      gnome_pixmap_entry_get_type    (void) G_GNUC_CONST;
 GtkWidget *gnome_pixmap_entry_new         (const gchar *history_id,
 					   const gchar *browse_dialog_title,
 					   gboolean do_preview);
-void       gnome_pixmap_entry_construct   (GnomePixmapEntry *pentry,
+void       gnome_pixmap_entry_construct   (GnomePixmapEntry *gentry,
 					   const gchar *history_id,
 					   const gchar *browse_dialog_title,
 					   gboolean do_preview);
