@@ -25,7 +25,7 @@
 */
 
 #include <config.h>
-#include <libgnome/gnome-macros.h>
+#include "gnome-macros.h"
 
 #ifndef GNOME_DISABLE_DEPRECATED_SOURCE
 
@@ -36,7 +36,7 @@
 
 
 static void gnome_pixmap_class_init    (GnomePixmapClass *class);
-static void gnome_pixmap_instance_init (GnomePixmap      *gpixmap);
+static void gnome_pixmap_init          (GnomePixmap      *gpixmap);
 
 /* Not used currently */
 struct _GnomePixmapPrivate {
@@ -52,14 +52,14 @@ struct _GnomePixmapPrivate {
  * Returns: the type ID of the &GnomePixmap class.
  */
 GNOME_CLASS_BOILERPLATE (GnomePixmap, gnome_pixmap,
-			 GtkImage, GTK_TYPE_IMAGE)
+			 GtkImage, gtk_image)
 
 /*
  * Widget functions
  */
 
 static void
-gnome_pixmap_instance_init (GnomePixmap *gpixmap)
+gnome_pixmap_init (GnomePixmap *gpixmap)
 {
 }
 
@@ -97,7 +97,7 @@ gnome_pixmap_class_init (GnomePixmapClass *class)
 GtkWidget*
 gnome_pixmap_new_from_file (const char *filename)
 {
-	GtkWidget *retval = gtk_type_new (GNOME_TYPE_PIXMAP);
+	GtkWidget *retval = gtk_type_new (gnome_pixmap_get_type ());
 	gtk_image_set_from_file (GTK_IMAGE (retval), filename);
 	return retval;
 }
@@ -136,12 +136,12 @@ gnome_pixmap_new_from_file_at_size (const gchar *filename, gint width, gint heig
 							     width,
 							     height,
 							     GDK_INTERP_BILINEAR);
-		retval = gtk_type_new (GNOME_TYPE_PIXMAP);
+		retval = gtk_type_new (gnome_pixmap_get_type ());
 		gtk_image_set_from_pixbuf (GTK_IMAGE (retval), scaled);
 		gdk_pixbuf_unref (scaled);
 		gdk_pixbuf_unref (pixbuf);
 	} else {
-		retval = gtk_type_new (GNOME_TYPE_PIXMAP);
+		retval = gtk_type_new (gnome_pixmap_get_type ());
 	}
 	return retval;
 }
@@ -166,11 +166,11 @@ gnome_pixmap_new_from_xpm_d (const char **xpm_data)
 
 	pixbuf = gdk_pixbuf_new_from_xpm_data (xpm_data);
 	if (pixbuf != NULL) {
-		retval = gtk_type_new (GNOME_TYPE_PIXMAP);
+		retval = gtk_type_new (gnome_pixmap_get_type ());
 		gtk_image_set_from_pixbuf (GTK_IMAGE (retval), pixbuf);
 		gdk_pixbuf_unref (pixbuf);
 	} else {
-		retval = gtk_type_new (GNOME_TYPE_PIXMAP);
+		retval = gtk_type_new (gnome_pixmap_get_type ());
 	}
 	return retval;
 }
@@ -201,12 +201,12 @@ gnome_pixmap_new_from_xpm_d_at_size (const char **xpm_data, int width, int heigh
 							     width,
 							     height,
 							     GDK_INTERP_BILINEAR);
-		retval = gtk_type_new (GNOME_TYPE_PIXMAP);
+		retval = gtk_type_new (gnome_pixmap_get_type ());
 		gtk_image_set_from_pixbuf (GTK_IMAGE (retval), scaled);
 		gdk_pixbuf_unref (scaled);
 		gdk_pixbuf_unref (pixbuf);
 	} else {
-		retval = gtk_type_new (GNOME_TYPE_PIXMAP);
+		retval = gtk_type_new (gnome_pixmap_get_type ());
 	}
 	return retval;
 }
@@ -223,10 +223,10 @@ gnome_pixmap_new_from_gnome_pixmap (GnomePixmap *gpixmap)
 
 	pixbuf = gtk_image_get_pixbuf (GTK_IMAGE (gpixmap));
 	if (pixbuf != NULL) {
-		retval = gtk_type_new (GNOME_TYPE_PIXMAP);
+		retval = gtk_type_new (gnome_pixmap_get_type ());
 		gtk_image_set_from_pixbuf (GTK_IMAGE (retval), pixbuf);
 	} else {
-		retval = gtk_type_new (GNOME_TYPE_PIXMAP);
+		retval = gtk_type_new (gnome_pixmap_get_type ());
 	}
 	return retval;
 }
