@@ -27,7 +27,8 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
-#include <libgnome/gnome-i18n.h>
+/* Must be before all other gnome includes!! */
+#include "gnome-i18nP.h"
 
 #include <libgnomeui/gnome-stock-icons.h>
 #include "pixmaps/gnome-stock-pixbufs.h"
@@ -36,6 +37,8 @@
 #define GETTEXT_PACKAGE PACKAGE
 #endif
 
+GtkIconSize gnome_icon_size_toolbar;
+
 static void G_GNUC_UNUSED
 add_sized (GtkIconFactory *factory,
            const guchar   *inline_data,
@@ -101,6 +104,7 @@ get_default_icons (GtkIconFactory *factory)
     
     add_sized (factory, stock_timer, GTK_ICON_SIZE_BUTTON, GNOME_STOCK_TIMER);
     add_sized (factory, stock_timer_stopped, GTK_ICON_SIZE_BUTTON, GNOME_STOCK_TIMER_STOP);
+    add_sized (factory, stock_trash, GTK_ICON_SIZE_BUTTON, GNOME_STOCK_TRASH);
     add_sized (factory, stock_trash_full, GTK_ICON_SIZE_BUTTON, GNOME_STOCK_TRASH_FULL);
 
     add_sized (factory, stock_mail, GTK_ICON_SIZE_BUTTON, GNOME_STOCK_MAIL);
