@@ -516,7 +516,6 @@ create_druid(void)
 	gnome_druid_set_page (GNOME_DRUID (druid), GNOME_DRUID_PAGE (page_start));
 	gtk_widget_show_all (window);
 }
-
 #endif
 
 /*
@@ -530,9 +529,8 @@ create_entry(void)
 	GtkWidget *entry;
 
 	app = create_newwin(TRUE,"testGNOME","Entry");
-	entry = gnome_entry_new();
+	entry = gnome_entry_new(NULL);
 	g_assert(entry != NULL);
-//	gnome_entry_set_text (GNOME_ENTRY(entry), "Foo");
 	bonobo_window_set_contents(BONOBO_WINDOW(app->app), entry);
 	gtk_widget_show(entry);
 	gtk_widget_show(app->app);
@@ -542,8 +540,6 @@ create_entry(void)
  * FileEntry
  */
 
-#if 0
-
 static void
 file_entry_update_files(GtkWidget *w, GnomeFileEntry *fentry)
 {
@@ -575,7 +571,7 @@ file_entry_modal_toggle(GtkWidget *w, GnomeFileEntry *fentry)
 static void
 file_entry_directory_toggle(GtkWidget *w, GnomeFileEntry *fentry)
 {
-	gnome_file_entry_set_directory(fentry,GTK_TOGGLE_BUTTON(w)->active);
+	gnome_file_entry_set_directory_entry (fentry,GTK_TOGGLE_BUTTON(w)->active);
 }
 
 static void
@@ -623,8 +619,6 @@ create_file_entry(void)
 	gtk_widget_show_all(app->app);
 }
 
-#endif
-
 /*
  * FontPicker
  */
@@ -917,6 +911,7 @@ create_icon_list(void)
 /*
  * ImageEntry
  */
+#if 0
 
 static void
 create_image_entry(void)
@@ -925,12 +920,13 @@ create_image_entry(void)
 	GtkWidget *entry;
 
 	app = create_newwin(TRUE,"testGNOME","Pixmap Entry");
-	/* FIXME: The gnome_image_entry_new param. must be reviewed */
-	entry = gnome_image_entry_new_pixmap_entry (0,0);
+	entry = gnome_pixmap_entry_new (NULL, "Test", FALSE);
 	bonobo_window_set_contents(BONOBO_WINDOW(app->app),entry);
 	gtk_widget_show(entry);
 	gtk_widget_show(app->app);
 }
+#endif
+
 #if 0
 /*
  * ImageSelector
@@ -982,13 +978,13 @@ main (int argc, char **argv)
 		{ "color picker", create_color_picker },
 		{ "date edit", create_date_edit },
 /*		{ "ditem edit", create_ditem_edit },*/
-/*		{ "druid", create_druid },*/
+/*		{ "druid", create_druid }, */
 		{ "entry", create_entry },
-/*		{ "file entry", create_file_entry },*/
+		{ "file entry", create_file_entry },
 /*		{ "font picker", create_font_picker },*/
 		{ "href", create_href },
 		{ "icon list", create_icon_list },
-		{ "image entry", create_image_entry },
+/*		{ "image entry", create_image_entry }, */
 /*		{ "image selector", create_image_selector },*/
 /*		{ "paper selector", create_papersel },*/
 /*		{ "unit spinner", create_unit_spinner },*/
@@ -1002,7 +998,8 @@ main (int argc, char **argv)
 	int i;
 
 	
-	gnome_program_init ("testGNOME", VERSION, &libgnomeui_module_info,
+	gnome_program_init ("testGNOME", VERSION,
+			    LIBGNOMEUI_MODULE,
 			    argc, argv, NULL);
 
 	app = create_newwin (FALSE, "testGNOME", "testGNOME");
